@@ -4,7 +4,7 @@ import sys
 import time
 from datetime import datetime
 from decimal import Decimal
-from typing import Any, Callable, List, Optional, Tuple, Dict
+from typing import Any, Awaitable, Callable, Dict, List, Optional, Tuple
 
 import aiohttp
 
@@ -24,6 +24,8 @@ from chia.wallet.trading.trade_status import TradeStatus
 from chia.wallet.transaction_record import TransactionRecord
 from chia.wallet.util.wallet_types import WalletType
 
+CATNameResolver = Callable[[bytes32], Awaitable[Optional[Tuple[Optional[uint32], str]]]]
+
 
 def print_transaction(tx: TransactionRecord, verbose: bool, name, address_prefix: str, mojo_per_unit: int) -> None:
     if verbose:
@@ -216,7 +218,8 @@ async def send(args: dict, wallet_client: WalletRpcClient, fingerprint: int) ->
 
 async def get_address(args: dict, wallet_client: WalletRpcClient, fingerprint: int) -> None:
     wallet_id = args["id"]
-    res = await wallet_client.get_next_address(wallet_id, False)
+    new_address: bool = args.get("new_address", False)
+    res = await wallet_client.get_next_address(wallet_id, new_address)
     print(res)
 
 
@@ -307,21 +310,37 @@ def timestamp_to_time(timestamp):
     return datetime.fromtimestamp(timestamp).strftime("%Y-%m-%d %H:%M:%S")
 
 
-async def print_offer_summary(wallet_client: WalletRpcClient, sum_dict: dict):
+async def print_offer_summary(cat_name_resolver: CATNameResolver, sum_dict: Dict[str, int], has_fee: bool = False):
     for asset_id, amount in sum_dict.items():
-        if asset_id == "xch":
-            wid: str = "1"
-            name: str = "XCH"
-            unit: int = units["chia"]
-        else:
-            result = await wallet_client.cat_asset_id_to_name(bytes32.from_hexstr(asset_id))
-            wid = "Unknown"
+        description: str = ""
+        unit: int = units["chia"]
+        wid: str = "1" if asset_id == "xch" else ""
+        mojo_amount: int = int(Decimal(amount))
+        name: str = "XCH"
+        if asset_id != "xch":
             name = asset_id
-            unit = units["cat"]
-            if result is not None:
-                wid = str(result[0])
-                name = result[1]
-        print(f"    - {name} (Wallet ID: {wid}): {Decimal(int(amount)) / unit} ({int(Decimal(amount))} mojos)")
+            if asset_id == "unknown":
+                name = "Unknown"
+                unit = units["mojo"]
+                if has_fee:
+                    description = " [Typically represents change returned from the included fee]"
+            else:
+                unit = units["cat"]
+                result = await cat_name_resolver(bytes32.from_hexstr(asset_id))
+                if result is not None:
+                    wid = str(result[0])
+                    name = result[1]
+        output: str = f"    - {name}"
+        mojo_str: str = f"{mojo_amount} {'mojo' if mojo_amount == 1 else 'mojos'}"
+        if len(wid) > 0:
+            output += f" (Wallet ID: {wid})"
+        if unit == units["mojo"]:
+            output += f": {mojo_str}"
+        else:
+            output += f": {mojo_amount / unit} ({mojo_str})"
+        if len(description) > 0:
+            output += f" {description}"
+        print(output)
 
 
 async def print_trade_record(record, wallet_client: WalletRpcClient, summaries: bool = False) -> None:
@@ -336,13 +355,16 @@ async def print_trade_record(record, wallet_client: WalletRpcClient, summaries:
         print("Summary:")
         offer = Offer.from_bytes(record.offer)
         offered, requested = offer.summary()
+        outbound_balances: Dict[str, int] = offer.get_pending_amounts()
+        fees: Decimal = Decimal(offer.bundle.fees())
+        cat_name_resolver = wallet_client.cat_asset_id_to_name
         print("  OFFERED:")
-        await print_offer_summary(wallet_client, offered)
+        await print_offer_summary(cat_name_resolver, offered)
         print("  REQUESTED:")
-        await print_offer_summary(wallet_client, requested)
-        print("Pending Balances:")
-        await print_offer_summary(wallet_client, offer.get_pending_amounts())
-        print(f"Fees: {Decimal(offer.bundle.fees()) / units['chia']}")
+        await print_offer_summary(cat_name_resolver, requested)
+        print("Pending Outbound Balances:")
+        await print_offer_summary(cat_name_resolver, outbound_balances, has_fee=(fees > 0))
+        print(f"Included Fees: {fees / units['chia']}")
     print("---------------")
 
 
@@ -410,12 +432,13 @@ async def take_offer(args: dict, wallet_client: WalletRpcClient, fingerprint: in
         return
 
     offered, requested = offer.summary()
+    cat_name_resolver = wallet_client.cat_asset_id_to_name
     print("Summary:")
     print("  OFFERED:")
-    await print_offer_summary(wallet_client, offered)
+    await print_offer_summary(cat_name_resolver, offered)
     print("  REQUESTED:")
-    await print_offer_summary(wallet_client, requested)
-    print(f"Fees: {Decimal(offer.bundle.fees()) / units['chia']}")
+    await print_offer_summary(cat_name_resolver, requested)
+    print(f"Included Fees: {Decimal(offer.bundle.fees()) / units['chia']}")
 
     if not examine_only:
         confirmation = input("Would you like to take this offer? (y/n): ")
@@ -457,7 +480,10 @@ def print_balance(amount: int, scale: int, address_prefix: str) -> str:
 
 
 async def print_balances(args: dict, wallet_client: WalletRpcClient, fingerprint: int) -> None:
-    summaries_response = await wallet_client.get_wallets()
+    wallet_type: Optional[WalletType] = None
+    if "type" in args:
+        wallet_type = WalletType(args["type"])
+    summaries_response = await wallet_client.get_wallets(wallet_type)
     config = load_config(DEFAULT_ROOT_PATH, "config.yaml")
     address_prefix = config["network_overrides"]["config"][config["selected_network"]]["address_prefix"]
 
@@ -473,22 +499,37 @@ async def print_balances(args: dict, wallet_client: WalletRpcClient, fingerprint
         print("Sync status: Not synced")
 
     if not is_syncing and is_synced:
-        print(f"Balances, fingerprint: {fingerprint}")
+        if len(summaries_response) == 0:
+            type_hint = " " if wallet_type is None else f" from type {wallet_type.name} "
+            print(f"\nNo wallets{type_hint}available for fingerprint: {fingerprint}")
+        else:
+            print(f"Balances, fingerprint: {fingerprint}")
         for summary in summaries_response:
+            indent: str = "   "
+            # asset_id currently contains both the asset ID and TAIL program bytes concatenated together.
+            # A future RPC update may split them apart, but for now we'll show the first 32 bytes (64 chars)
+            asset_id = summary["data"][:64]
             wallet_id = summary["id"]
             balances = await wallet_client.get_wallet_balance(wallet_id)
             typ = WalletType(int(summary["type"]))
             address_prefix, scale = wallet_coin_unit(typ, address_prefix)
-            print(f"Wallet ID {wallet_id} type {typ.name} {summary['name']}")
-            print(f"   -Total Balance: {print_balance(balances['confirmed_wallet_balance'], scale, address_prefix)}")
-            print(
-                f"   -Pending Total Balance: "
-                f"{print_balance(balances['unconfirmed_wallet_balance'], scale, address_prefix)}"
+            total_balance: str = print_balance(balances["confirmed_wallet_balance"], scale, address_prefix)
+            unconfirmed_wallet_balance: str = print_balance(
+                balances["unconfirmed_wallet_balance"], scale, address_prefix
             )
-            print(f"   -Spendable: {print_balance(balances['spendable_balance'], scale, address_prefix)}")
+            spendable_balance: str = print_balance(balances["spendable_balance"], scale, address_prefix)
+            print()
+            print(f"{summary['name']}:")
+            print(f"{indent}{'-Total Balance:'.ljust(23)} {total_balance}")
+            print(f"{indent}{'-Pending Total Balance:'.ljust(23)} " f"{unconfirmed_wallet_balance}")
+            print(f"{indent}{'-Spendable:'.ljust(23)} {spendable_balance}")
+            print(f"{indent}{'-Type:'.ljust(23)} {typ.name}")
+            if len(asset_id) > 0:
+                print(f"{indent}{'-Asset ID:'.ljust(23)} {asset_id}")
+            print(f"{indent}{'-Wallet ID:'.ljust(23)} {wallet_id}")
 
     print(" ")
-    trusted_peers: Dict = config.get("trusted_peers", {})
+    trusted_peers: Dict = config["wallet"].get("trusted_peers", {})
     await print_connections(wallet_client, time, NodeType, trusted_peers)
 
 
@@ -505,19 +546,40 @@ async def get_wallet(wallet_client: WalletRpcClient, fingerprint: int = None) ->
     if fingerprint is not None:
         log_in_response = await wallet_client.log_in(fingerprint)
     else:
-        print("Choose wallet key:")
+        logged_in_fingerprint: Optional[int] = await wallet_client.get_logged_in_fingerprint()
+        spacing: str = "  " if logged_in_fingerprint is not None else ""
+        current_sync_status: str = ""
+        if logged_in_fingerprint is not None:
+            if await wallet_client.get_synced():
+                current_sync_status = "Synced"
+            elif await wallet_client.get_sync_status():
+                current_sync_status = "Syncing"
+            else:
+                current_sync_status = "Not Synced"
+        print("Wallet keys:")
         for i, fp in enumerate(fingerprints):
-            print(f"{i+1}) {fp}")
+            row: str = f"{i+1}) "
+            row += "* " if fp == logged_in_fingerprint else spacing
+            row += f"{fp}"
+            if fp == logged_in_fingerprint and len(current_sync_status) > 0:
+                row += f" ({current_sync_status})"
+            print(row)
         val = None
+        prompt: str = (
+            f"Choose a wallet key [1-{len(fingerprints)}] ('q' to quit, or Enter to use {logged_in_fingerprint}): "
+        )
         while val is None:
-            val = input("Enter a number to pick or q to quit: ")
+            val = input(prompt)
             if val == "q":
                 return None
-            if not val.isdigit():
+            elif val == "" and logged_in_fingerprint is not None:
+                fingerprint = logged_in_fingerprint
+                break
+            elif not val.isdigit():
                 val = None
             else:
                 index = int(val) - 1
-                if index >= len(fingerprints):
+                if index < 0 or index >= len(fingerprints):
                     print("Invalid value")
                     val = None
                     continue
