@@ -1,9 +1,8 @@
 import collections
 import logging
-from typing import Callable, Dict, List, Optional, Set, Tuple, Union
+from typing import Awaitable, Callable, Dict, List, Optional, Set, Tuple, Union
 
 from chiabip158 import PyBIP158
-from clvm.casts import int_from_bytes
 
 from chia.consensus.block_record import BlockRecord
 from chia.consensus.block_rewards import calculate_base_farmer_reward, calculate_pool_reward
@@ -15,22 +14,20 @@ from chia.consensus.cost_calculator import NPCResult
 from chia.consensus.find_fork_point import find_fork_point_in_chain
 from chia.full_node.block_store import BlockStore
 from chia.full_node.coin_store import CoinStore
-from chia.full_node.mempool_check_conditions import get_name_puzzle_conditions, mempool_check_conditions_dict
+from chia.full_node.mempool_check_conditions import get_name_puzzle_conditions, mempool_check_time_locks
+from chia.types.block_protocol import BlockInfo
 from chia.types.blockchain_format.coin import Coin
-from chia.types.blockchain_format.sized_bytes import bytes32
+from chia.types.blockchain_format.sized_bytes import bytes32, bytes48
 from chia.types.coin_record import CoinRecord
-from chia.types.condition_opcodes import ConditionOpcode
-from chia.types.condition_with_args import ConditionWithArgs
 from chia.types.full_block import FullBlock
 from chia.types.generator_types import BlockGenerator
-from chia.types.name_puzzle_condition import NPC
 from chia.types.unfinished_block import UnfinishedBlock
 from chia.util import cached_bls
 from chia.util.condition_tools import pkm_pairs
 from chia.util.errors import Err
-from chia.util.generator_tools import additions_for_npc, tx_removals_and_additions
+from chia.util.generator_tools import tx_removals_and_additions
 from chia.util.hash import std_hash
-from chia.util.ints import uint32, uint64, uint128
+from chia.util.ints import uint32, uint64
 
 log = logging.getLogger(__name__)
 
@@ -45,7 +42,7 @@ async def validate_block_body(
     height: uint32,
     npc_result: Optional[NPCResult],
     fork_point_with_peak: Optional[uint32],
-    get_block_generator: Callable,
+    get_block_generator: Callable[[BlockInfo], Awaitable[Optional[BlockGenerator]]],
     *,
     validate_signature=True,
 ) -> Tuple[Optional[Err], Optional[NPCResult]]:
@@ -152,7 +149,6 @@ async def validate_block_body(
     removals: List[bytes32] = []
     coinbase_additions: List[Coin] = list(expected_reward_coins)
     additions: List[Coin] = []
-    npc_list: List[NPC] = []
     removals_puzzle_dic: Dict[bytes32, bytes32] = {}
     cost: uint64 = uint64(0)
 
@@ -195,7 +191,6 @@ async def validate_block_body(
 
         assert npc_result is not None
         cost = npc_result.cost
-        npc_list = npc_result.npc_list
 
         # 7. Check that cost <= MAX_BLOCK_COST_CLVM
         log.debug(
@@ -209,11 +204,13 @@ async def validate_block_body(
         if npc_result.error is not None:
             return Err(npc_result.error), None
 
-        for npc in npc_list:
-            removals.append(npc.coin_name)
-            removals_puzzle_dic[npc.coin_name] = npc.puzzle_hash
+        assert npc_result.conds is not None
 
-        additions = additions_for_npc(npc_list)
+        for spend in npc_result.conds.spends:
+            removals.append(spend.coin_id)
+            removals_puzzle_dic[spend.coin_id] = spend.puzzle_hash
+            for puzzle_hash, amount, _ in spend.create_coin:
+                additions.append(Coin(spend.coin_id, puzzle_hash, uint64(amount)))
     else:
         assert npc_result is None
 
@@ -244,18 +241,12 @@ async def validate_block_body(
         return root_error, None
 
     # 12. The additions and removals must result in the correct filter
-    byte_array_tx: List[bytes32] = []
+    byte_array_tx: List[bytearray] = []
 
     for coin in additions + coinbase_additions:
-        # TODO: address hint error and remove ignore
-        #       error: Argument 1 to "append" of "list" has incompatible type "bytearray"; expected "bytes32"
-        #       [arg-type]
-        byte_array_tx.append(bytearray(coin.puzzle_hash))  # type: ignore[arg-type]
+        byte_array_tx.append(bytearray(coin.puzzle_hash))
     for coin_name in removals:
-        # TODO: address hint error and remove ignore
-        #       error: Argument 1 to "append" of "list" has incompatible type "bytearray"; expected "bytes32"
-        #       [arg-type]
-        byte_array_tx.append(bytearray(coin_name))  # type: ignore[arg-type]
+        byte_array_tx.append(bytearray(coin_name))
 
     bip158: PyBIP158 = PyBIP158(byte_array_tx)
     encoded_filter = bytes(bip158.GetEncoded())
@@ -322,7 +313,7 @@ async def validate_block_body(
                     mempool_mode=False,
                     height=curr.height,
                 )
-                removals_in_curr, additions_in_curr = tx_removals_and_additions(curr_npc_result.npc_list)
+                removals_in_curr, additions_in_curr = tx_removals_and_additions(curr_npc_result.conds)
             else:
                 removals_in_curr = []
                 additions_in_curr = []
@@ -345,7 +336,7 @@ async def validate_block_body(
                 )
             if curr.height == 0:
                 break
-            curr = reorg_blocks[curr.height - 1]
+            curr = reorg_blocks[uint32(curr.height - 1)]
             assert curr is not None
 
     removal_coin_records: Dict[bytes32, CoinRecord] = {}
@@ -406,16 +397,13 @@ async def validate_block_body(
 
     fees = removed - added
     assert fees >= 0
-    assert_fee_sum: uint128 = uint128(0)
 
-    for npc in npc_list:
-        if ConditionOpcode.RESERVE_FEE in npc.condition_dict:
-            fee_list: List[ConditionWithArgs] = npc.condition_dict[ConditionOpcode.RESERVE_FEE]
-            for cvp in fee_list:
-                fee = int_from_bytes(cvp.vars[0])
-                if fee < 0:
-                    return Err.RESERVE_FEE_CONDITION_FAILED, None
-                assert_fee_sum = uint128(assert_fee_sum + fee)
+    # reserve fee cannot be greater than UINT64_MAX per consensus rule.
+    # run_generator() would fail
+    assert_fee_sum: uint64 = uint64(0)
+    if npc_result:
+        assert npc_result.conds is not None
+        assert_fee_sum = npc_result.conds.reserve_fee
 
     # 17. Check that the assert fee sum <= fees, and that each reserved fee is non-negative
     if fees < assert_fee_sum:
@@ -435,12 +423,12 @@ async def validate_block_body(
             return Err.WRONG_PUZZLE_HASH, None
 
     # 21. Verify conditions
-    for npc in npc_list:
-        assert height is not None
-        unspent = removal_coin_records[npc.coin_name]
-        error = mempool_check_conditions_dict(
-            unspent,
-            npc.condition_dict,
+    # verify absolute/relative height/time conditions
+    if npc_result is not None:
+        assert npc_result.conds is not None
+        error = mempool_check_time_locks(
+            removal_coin_records,
+            npc_result.conds,
             prev_transaction_block_height,
             block.foliage_transaction_block.timestamp,
         )
@@ -448,7 +436,11 @@ async def validate_block_body(
             return error, None
 
     # create hash_key list for aggsig check
-    pairs_pks, pairs_msgs = pkm_pairs(npc_list, constants.AGG_SIG_ME_ADDITIONAL_DATA)
+    pairs_pks: List[bytes48] = []
+    pairs_msgs: List[bytes] = []
+    if npc_result:
+        assert npc_result.conds is not None
+        pairs_pks, pairs_msgs = pkm_pairs(npc_result.conds, constants.AGG_SIG_ME_ADDITIONAL_DATA)
 
     # 22. Verify aggregated signature
     # TODO: move this to pre_validate_blocks_multiprocessing so we can sync faster
