@@ -17,14 +17,14 @@ class TimelordAPI:
         self.timelord = timelord
 
     def _set_state_changed_callback(self, callback: Callable):
-        pass
+        self.timelord.state_changed_callback = callback
 
     @api_request
     async def new_peak_timelord(self, new_peak: timelord_protocol.NewPeakTimelord):
         if self.timelord.last_state is None:
             return None
         async with self.timelord.lock:
-            if self.timelord.sanitizer_mode:
+            if self.timelord.bluebox_mode:
                 return None
             if new_peak.reward_chain_block.weight > self.timelord.last_state.get_weight():
                 log.info("Not skipping peak, don't have. Maybe we are not the fastest timelord")
@@ -33,15 +33,18 @@ class TimelordAPI:
                     f"{new_peak.reward_chain_block.weight} "
                 )
                 self.timelord.new_peak = new_peak
+                self.timelord.state_changed("new_peak", {"height": new_peak.reward_chain_block.height})
             elif (
                 self.timelord.last_state.peak is not None
                 and self.timelord.last_state.peak.reward_chain_block == new_peak.reward_chain_block
             ):
                 log.info("Skipping peak, already have.")
+                self.timelord.state_changed("skipping_peak", {"height": new_peak.reward_chain_block.height})
                 return None
             else:
                 log.warning("block that we don't have, changing to it.")
                 self.timelord.new_peak = new_peak
+                self.timelord.state_changed("new_peak", {"height": new_peak.reward_chain_block.height})
                 self.timelord.new_subslot_end = None
 
     @api_request
@@ -49,7 +52,7 @@ class TimelordAPI:
         if self.timelord.last_state is None:
             return None
         async with self.timelord.lock:
-            if self.timelord.sanitizer_mode:
+            if self.timelord.bluebox_mode:
                 return None
             try:
                 sp_iters, ip_iters = iters_from_block(
@@ -79,7 +82,7 @@ class TimelordAPI:
     @api_request
     async def request_compact_proof_of_time(self, vdf_info: timelord_protocol.RequestCompactProofOfTime):
         async with self.timelord.lock:
-            if not self.timelord.sanitizer_mode:
+            if not self.timelord.bluebox_mode:
                 return None
             now = time.time()
             # work older than 5s can safely be assumed to be from the previous batch, and needs to be cleared
