@@ -1,39 +1,21 @@
 import io
-from typing import List, Set, Tuple, Optional, Any
+from typing import List, Set, Tuple, Optional
 
 from clvm import SExp
-from clvm import run_program as default_run_program
 from clvm.casts import int_from_bytes
 from clvm.EvalError import EvalError
-from clvm.operators import OPERATOR_LOOKUP
 from clvm.serialize import sexp_from_stream, sexp_to_stream
-from clvm_rs import MEMPOOL_MODE, run_chia_program, serialized_length, run_generator2
+from chia_rs import MEMPOOL_MODE, run_chia_program, serialized_length, run_generator
 from clvm_tools.curry import curry, uncurry
 
 from chia.types.blockchain_format.sized_bytes import bytes32
 from chia.util.hash import std_hash
-from chia.util.ints import uint16
 from chia.util.byte_types import hexstr_to_bytes
+from chia.types.spend_bundle_conditions import SpendBundleConditions, Spend
 
 from .tree_hash import sha256_treehash
 
 
-def run_program(
-    program,
-    args,
-    max_cost,
-    operator_lookup=OPERATOR_LOOKUP,
-    pre_eval_f=None,
-):
-    return default_run_program(
-        program,
-        args,
-        operator_lookup,
-        max_cost,
-        pre_eval_f=pre_eval_f,
-    )
-
-
 INFINITE_COST = 0x7FFFFFFFFFFFFFFF
 
 
@@ -63,10 +45,6 @@ class Program(SExp):
     def to_serialized_program(self) -> "SerializedProgram":
         return SerializedProgram.from_bytes(bytes(self))
 
-    @classmethod
-    def from_serialized_program(cls, sp: "SerializedProgram") -> "Program":
-        return cls.from_bytes(bytes(sp))
-
     def __bytes__(self) -> bytes:
         f = io.BytesIO()
         self.stream(f)  # noqa
@@ -103,7 +81,7 @@ class Program(SExp):
 
     def run_with_cost(self, max_cost: int, args) -> Tuple[int, "Program"]:
         prog_args = Program.to(args)
-        cost, r = run_program(self, prog_args, max_cost)
+        cost, r = run_chia_program(self.as_bin(), prog_args.as_bin(), max_cost, 0)
         return cost, Program.to(r)
 
     def run(self, args) -> "Program":
@@ -244,9 +222,12 @@ class SerializedProgram:
     def run_with_cost(self, max_cost: int, *args) -> Tuple[int, Program]:
         return self._run(max_cost, 0, *args)
 
-    # returns an optional error code and an optional PySpendBundleConditions (from clvm_rs)
+    # returns an optional error code and an optional SpendBundleConditions (from chia_rs)
     # exactly one of those will hold a value
-    def run_as_generator(self, max_cost: int, flags: int, *args) -> Tuple[Optional[uint16], Optional[Any]]:
+    def run_as_generator(
+        self, max_cost: int, flags: int, *args
+    ) -> Tuple[Optional[int], Optional[SpendBundleConditions]]:
+
         serialized_args = b""
         if len(args) > 1:
             # when we have more than one argument, serialize them into a list
@@ -257,12 +238,31 @@ class SerializedProgram:
         else:
             serialized_args += _serialize(args[0])
 
-        return run_generator2(
+        err, conds = run_generator(
             self._buf,
             serialized_args,
             max_cost,
             flags,
         )
+        if err is not None:
+            assert err != 0
+            return err, None
+
+        # for now, we need to copy this data into python objects, in order to
+        # support streamable. This will become simpler and faster once we can
+        # implement streamable in rust
+        spends = []
+        for s in conds.spends:
+            spends.append(
+                Spend(s.coin_id, s.puzzle_hash, s.height_relative, s.seconds_relative, s.create_coin, s.agg_sig_me)
+            )
+
+        ret = SpendBundleConditions(
+            spends, conds.reserve_fee, conds.height_absolute, conds.seconds_absolute, conds.agg_sig_unsafe, conds.cost
+        )
+
+        assert ret is not None
+        return None, ret
 
     def _run(self, max_cost: int, flags, *args) -> Tuple[int, Program]:
         # when multiple arguments are passed, concatenate them into a serialized
