@@ -5,7 +5,7 @@ import io
 import pprint
 import sys
 from enum import Enum
-from typing import Any, BinaryIO, Dict, get_type_hints, List, Tuple, Type, TypeVar, Callable, Optional, Iterator
+from typing import Any, BinaryIO, Dict, get_type_hints, List, Tuple, Type, TypeVar, Union, Callable, Optional, Iterator
 
 from blspy import G1Element, G2Element, PrivateKey
 from typing_extensions import Literal
@@ -14,20 +14,31 @@ from chia.types.blockchain_format.sized_bytes import bytes32
 from chia.util.byte_types import hexstr_to_bytes
 from chia.util.hash import std_hash
 from chia.util.ints import int64, int512, uint32, uint64, uint128
-from chia.util.type_checking import is_type_List, is_type_SpecificOptional, is_type_Tuple, strictdataclass
 
 if sys.version_info < (3, 8):
 
     def get_args(t: Type[Any]) -> Tuple[Any, ...]:
         return getattr(t, "__args__", ())
 
+    def get_origin(t: Type[Any]) -> Optional[Type[Any]]:
+        return getattr(t, "__origin__", None)
+
 else:
 
-    from typing import get_args
+    from typing import get_args, get_origin
 
 
 pp = pprint.PrettyPrinter(indent=1, width=120, compact=True)
 
+
+class StreamableError(Exception):
+    pass
+
+
+class DefinitionError(StreamableError):
+    pass
+
+
 # TODO: Remove hack, this allows streaming these objects from binary
 size_hints = {
     "PrivateKey": PrivateKey.PRIVATE_KEY_SIZE,
@@ -48,6 +59,27 @@ big_ints = [uint64, int64, uint128, int512]
 _T_Streamable = TypeVar("_T_Streamable", bound="Streamable")
 
 
+# Caches to store the fields and (de)serialization methods for all available streamable classes.
+FIELDS_FOR_STREAMABLE_CLASS = {}
+STREAM_FUNCTIONS_FOR_STREAMABLE_CLASS = {}
+PARSE_FUNCTIONS_FOR_STREAMABLE_CLASS = {}
+
+
+def is_type_List(f_type: Type) -> bool:
+    return get_origin(f_type) == list or f_type == list
+
+
+def is_type_SpecificOptional(f_type) -> bool:
+    """
+    Returns true for types such as Optional[T], but not Optional, or T.
+    """
+    return get_origin(f_type) == Union and get_args(f_type)[1]() is None
+
+
+def is_type_Tuple(f_type: Type) -> bool:
+    return get_origin(f_type) == tuple or f_type == tuple
+
+
 def dataclass_from_dict(klass, d):
     """
     Converts a dictionary based on a dataclass, into an instance of that dataclass.
@@ -55,7 +87,7 @@ def dataclass_from_dict(klass, d):
     """
     if is_type_SpecificOptional(klass):
         # Type is optional, data is either None, or Any
-        if not d:
+        if d is None:
             return None
         return dataclass_from_dict(get_args(klass)[0], d)
     elif is_type_Tuple(klass):
@@ -124,81 +156,6 @@ def recurse_jsonify(d):
     return d
 
 
-STREAM_FUNCTIONS_FOR_STREAMABLE_CLASS = {}
-PARSE_FUNCTIONS_FOR_STREAMABLE_CLASS = {}
-FIELDS_FOR_STREAMABLE_CLASS = {}
-
-
-def streamable(cls: Any):
-    """
-    This is a decorator for class definitions. It applies the strictdataclass decorator,
-    which checks all types at construction. It also defines a simple serialization format,
-    and adds parse, from bytes, stream, and __bytes__ methods.
-
-    The primitives are:
-    * Sized ints serialized in big endian format, e.g. uint64
-    * Sized bytes serialized in big endian format, e.g. bytes32
-    * BLS public keys serialized in bls format (48 bytes)
-    * BLS signatures serialized in bls format (96 bytes)
-    * bool serialized into 1 byte (0x01 or 0x00)
-    * bytes serialized as a 4 byte size prefix and then the bytes.
-    * ConditionOpcode is serialized as a 1 byte value.
-    * str serialized as a 4 byte size prefix and then the utf-8 representation in bytes.
-
-    An item is one of:
-    * primitive
-    * Tuple[item1, .. itemx]
-    * List[item1, .. itemx]
-    * Optional[item]
-    * Custom item
-
-    A streamable must be a Tuple at the root level (although a dataclass is used here instead).
-    Iters are serialized in the following way:
-
-    1. A tuple of x items is serialized by appending the serialization of each item.
-    2. A List is serialized into a 4 byte size prefix (number of items) and the serialization of each item.
-    3. An Optional is serialized into a 1 byte prefix of 0x00 or 0x01, and if it's one, it's followed by the
-       serialization of the item.
-    4. A Custom item is serialized by calling the .parse method, passing in the stream of bytes into it. An example is
-       a CLVM program.
-
-    All of the constituents must have parse/from_bytes, and stream/__bytes__ and therefore
-    be of fixed size. For example, int cannot be a constituent since it is not a fixed size,
-    whereas uint32 can be.
-
-    Furthermore, a get_hash() member is added, which performs a serialization and a sha256.
-
-    This class is used for deterministic serialization and hashing, for consensus critical
-    objects such as the block header.
-
-    Make sure to use the Streamable class as a parent class when using the streamable decorator,
-    as it will allow linters to recognize the methods that are added by the decorator. Also,
-    use the @dataclass(frozen=True) decorator as well, for linters to recognize constructor
-    arguments.
-    """
-
-    cls1 = strictdataclass(cls)
-    t = type(cls.__name__, (cls1, Streamable), {})
-
-    stream_functions = []
-    parse_functions = []
-    try:
-        hints = get_type_hints(t)
-        fields = {field.name: hints.get(field.name, field.type) for field in dataclasses.fields(t)}
-    except Exception:
-        fields = {}
-
-    FIELDS_FOR_STREAMABLE_CLASS[t] = fields
-
-    for _, f_type in fields.items():
-        stream_functions.append(cls.function_to_stream_one_item(f_type))
-        parse_functions.append(cls.function_to_parse_one_item(f_type))
-
-    STREAM_FUNCTIONS_FOR_STREAMABLE_CLASS[t] = stream_functions
-    PARSE_FUNCTIONS_FOR_STREAMABLE_CLASS[t] = parse_functions
-    return t
-
-
 def parse_bool(f: BinaryIO) -> bool:
     bool_byte = f.read(1)
     assert bool_byte is not None and len(bool_byte) == 1  # Checks for EOF
@@ -298,7 +255,158 @@ def stream_str(item: Any, f: BinaryIO) -> None:
     f.write(str_bytes)
 
 
+def streamable(cls: Any):
+    """
+    This decorator forces correct streamable protocol syntax/usage and populates the caches for types hints and
+    (de)serialization methods for all members of the class. The correct usage is:
+
+    @streamable
+    @dataclass(frozen=True)
+    class Example(Streamable):
+        ...
+
+    The order how the decorator are applied and the inheritance from Streamable are forced. The explicit inheritance is
+    required because mypy doesn't analyse the type returned by decorators, so we can't just inherit from inside the
+    decorator. The dataclass decorator is required to fetch type hints, let mypy validate constructor calls and restrict
+    direct modification of objects by `frozen=True`.
+    """
+
+    correct_usage_string: str = (
+        "Correct usage is:\n\n@streamable\n@dataclass(frozen=True)\nclass Example(Streamable):\n    ..."
+    )
+
+    if not dataclasses.is_dataclass(cls):
+        raise DefinitionError(f"@dataclass(frozen=True) required first. {correct_usage_string}")
+
+    try:
+        object.__new__(cls)._streamable_test_if_dataclass_frozen_ = None
+    except dataclasses.FrozenInstanceError:
+        pass
+    else:
+        raise DefinitionError(f"dataclass needs to be frozen. {correct_usage_string}")
+
+    if not issubclass(cls, Streamable):
+        raise DefinitionError(f"Streamable inheritance required. {correct_usage_string}")
+
+    stream_functions = []
+    parse_functions = []
+    try:
+        hints = get_type_hints(cls)
+        fields = {field.name: hints.get(field.name, field.type) for field in dataclasses.fields(cls)}
+    except Exception:
+        fields = {}
+
+    FIELDS_FOR_STREAMABLE_CLASS[cls] = fields
+
+    for _, f_type in fields.items():
+        stream_functions.append(cls.function_to_stream_one_item(f_type))
+        parse_functions.append(cls.function_to_parse_one_item(f_type))
+
+    STREAM_FUNCTIONS_FOR_STREAMABLE_CLASS[cls] = stream_functions
+    PARSE_FUNCTIONS_FOR_STREAMABLE_CLASS[cls] = parse_functions
+    return cls
+
+
 class Streamable:
+    """
+    This class defines a simple serialization format, and adds methods to parse from/to bytes and json. It also
+    validates and parses all fields at construction in ´__post_init__` to make sure all fields have the correct type
+    and can be streamed/parsed properly.
+
+    The available primitives are:
+    * Sized ints serialized in big endian format, e.g. uint64
+    * Sized bytes serialized in big endian format, e.g. bytes32
+    * BLS public keys serialized in bls format (48 bytes)
+    * BLS signatures serialized in bls format (96 bytes)
+    * bool serialized into 1 byte (0x01 or 0x00)
+    * bytes serialized as a 4 byte size prefix and then the bytes.
+    * ConditionOpcode is serialized as a 1 byte value.
+    * str serialized as a 4 byte size prefix and then the utf-8 representation in bytes.
+
+    An item is one of:
+    * primitive
+    * Tuple[item1, .. itemx]
+    * List[item1, .. itemx]
+    * Optional[item]
+    * Custom item
+
+    A streamable must be a Tuple at the root level (although a dataclass is used here instead).
+    Iters are serialized in the following way:
+
+    1. A tuple of x items is serialized by appending the serialization of each item.
+    2. A List is serialized into a 4 byte size prefix (number of items) and the serialization of each item.
+    3. An Optional is serialized into a 1 byte prefix of 0x00 or 0x01, and if it's one, it's followed by the
+       serialization of the item.
+    4. A Custom item is serialized by calling the .parse method, passing in the stream of bytes into it. An example is
+       a CLVM program.
+
+    All of the constituents must have parse/from_bytes, and stream/__bytes__ and therefore
+    be of fixed size. For example, int cannot be a constituent since it is not a fixed size,
+    whereas uint32 can be.
+
+    Furthermore, a get_hash() member is added, which performs a serialization and a sha256.
+
+    This class is used for deterministic serialization and hashing, for consensus critical
+    objects such as the block header.
+
+    Make sure to use the streamable decorator when inheriting from the Streamable class to prepare the streaming caches.
+    """
+
+    def post_init_parse(self, item: Any, f_name: str, f_type: Type) -> Any:
+        if is_type_List(f_type):
+            collected_list: List = []
+            inner_type: Type = get_args(f_type)[0]
+            # wjb assert inner_type != get_args(List)[0]  # type: ignore
+            if not is_type_List(type(item)):
+                raise ValueError(f"Wrong type for {f_name}, need a list.")
+            for el in item:
+                collected_list.append(self.post_init_parse(el, f_name, inner_type))
+            return collected_list
+        if is_type_SpecificOptional(f_type):
+            if item is None:
+                return None
+            else:
+                inner_type: Type = get_args(f_type)[0]  # type: ignore
+                return self.post_init_parse(item, f_name, inner_type)
+        if is_type_Tuple(f_type):
+            collected_list = []
+            if not is_type_Tuple(type(item)) and not is_type_List(type(item)):
+                raise ValueError(f"Wrong type for {f_name}, need a tuple.")
+            if len(item) != len(get_args(f_type)):
+                raise ValueError(f"Wrong number of elements in tuple {f_name}.")
+            for i in range(len(item)):
+                inner_type = get_args(f_type)[i]
+                tuple_item = item[i]
+                collected_list.append(self.post_init_parse(tuple_item, f_name, inner_type))
+            return tuple(collected_list)
+        if not isinstance(item, f_type):
+            try:
+                item = f_type(item)
+            except (TypeError, AttributeError, ValueError):
+                try:
+                    item = f_type.from_bytes(item)
+                except Exception:
+                    item = f_type.from_bytes(bytes(item))
+        if not isinstance(item, f_type):
+            raise ValueError(f"Wrong type for {f_name}")
+        return item
+
+    def __post_init__(self):
+        try:
+            fields = FIELDS_FOR_STREAMABLE_CLASS[type(self)]
+        except Exception:
+            fields = {}
+        data = self.__dict__
+        for (f_name, f_type) in fields.items():
+            if f_name not in data:
+                raise ValueError(f"Field {f_name} not present")
+            try:
+                if not isinstance(data[f_name], f_type):
+                    object.__setattr__(self, f_name, self.post_init_parse(data[f_name], f_name, f_type))
+            except TypeError:
+                # Throws a TypeError because we cannot call isinstance for subscripted generics like Optional[int]
+                object.__setattr__(self, f_name, self.post_init_parse(data[f_name], f_name, f_type))
+
     @classmethod
     def function_to_parse_one_item(cls, f_type: Type) -> Callable[[BinaryIO], Any]:
         """
