@@ -1,6 +1,16 @@
 import io
 import struct
-from typing import Any, BinaryIO
+from typing import Any, BinaryIO, SupportsInt, Type, TypeVar, Union
+
+from typing_extensions import Protocol, SupportsIndex
+
+_T_StructStream = TypeVar("_T_StructStream", bound="StructStream")
+
+
+# https://github.com/python/typeshed/blob/c2182fdd3e572a1220c70ad9c28fd908b70fb19b/stdlib/_typeshed/__init__.pyi#L68-L69
+class SupportsTrunc(Protocol):
+    def __trunc__(self) -> int:
+        ...
 
 
 class StructStream(int):
@@ -10,7 +20,11 @@ class StructStream(int):
     Create a class that can parse and stream itself based on a struct.pack template string.
     """
 
-    def __new__(cls: Any, value: int):
+    # This is just a partial exposure of the underlying int constructor.  Liskov...
+    # https://github.com/python/typeshed/blob/5d07ebc864577c04366fcc46b84479dbec033921/stdlib/builtins.pyi#L181-L185
+    def __new__(
+        cls: Type[_T_StructStream], value: Union[str, bytes, SupportsInt, SupportsIndex, SupportsTrunc]
+    ) -> _T_StructStream:
         value = int(value)
         try:
             v1 = struct.unpack(cls.PACK, struct.pack(cls.PACK, value))[0]
@@ -21,7 +35,7 @@ class StructStream(int):
             raise ValueError(
                 f"Value {value} of size {value.bit_length()} does not fit into " f"{cls.__name__} of size {bits}"
             )
-        return int.__new__(cls, value)  # type: ignore
+        return int.__new__(cls, value)
 
     @classmethod
     def parse(cls: Any, f: BinaryIO) -> Any:
