@@ -4,7 +4,7 @@ from typing import Any, Dict, List, Optional, Set
 
 from blspy import G1Element
 
-from chia.consensus.cost_calculator import calculate_cost_of_program, NPCResult
+from chia.consensus.cost_calculator import NPCResult
 from chia.full_node.bundle_tools import simple_solution_generator
 from chia.full_node.mempool_check_conditions import get_name_puzzle_conditions
 from chia.types.blockchain_format.coin import Coin
@@ -37,9 +37,10 @@ from chia.wallet.secret_key_store import SecretKeyStore
 from chia.wallet.sign_coin_spends import sign_coin_spends
 from chia.wallet.transaction_record import TransactionRecord
 from chia.wallet.util.transaction_type import TransactionType
-from chia.wallet.util.wallet_types import WalletType
+from chia.wallet.util.wallet_types import WalletType, AmountWithPuzzlehash
 from chia.wallet.wallet_coin_record import WalletCoinRecord
 from chia.wallet.wallet_info import WalletInfo
+from chia.wallet.util.compute_memos import compute_memos
 
 
 class Wallet:
@@ -81,13 +82,9 @@ class Wallet:
                 program,
                 self.wallet_state_manager.constants.MAX_BLOCK_COST_CLVM,
                 cost_per_byte=self.wallet_state_manager.constants.COST_PER_BYTE,
-                safe_mode=True,
-                rust_checker=True,
+                mempool_mode=True,
             )
-            cost_result: uint64 = calculate_cost_of_program(
-                program.program, result, self.wallet_state_manager.constants.COST_PER_BYTE
-            )
-            self.cost_of_single_tx = cost_result
+            self.cost_of_single_tx = result.cost
             self.log.info(f"Cost of a single tx for standard wallet: {self.cost_of_single_tx}")
 
         max_cost = self.wallet_state_manager.constants.MAX_BLOCK_COST_CLVM / 5  # avoid full block TXs
@@ -123,12 +120,15 @@ class Wallet:
         return spendable
 
     async def get_pending_change_balance(self) -> uint64:
-        unconfirmed_tx = await self.wallet_state_manager.tx_store.get_unconfirmed_for_wallet(self.id())
+        unconfirmed_tx: List[TransactionRecord] = await self.wallet_state_manager.tx_store.get_unconfirmed_for_wallet(
+            self.id()
+        )
         addition_amount = 0
 
         for record in unconfirmed_tx:
             if not record.is_in_mempool():
-                self.log.warning(f"Record: {record} not in mempool")
+                if record.spend_bundle is not None:
+                    self.log.warning(f"Record: {record} not in mempool, {record.sent_to}")
                 continue
             our_spend = False
             for coin in record.removals:
@@ -148,6 +148,9 @@ class Wallet:
     def puzzle_for_pk(self, pubkey: bytes) -> Program:
         return puzzle_for_pk(pubkey)
 
+    async def convert_puzzle_hash(self, puzzle_hash: bytes32) -> bytes32:
+        return puzzle_hash  # Looks unimpressive, but it's more complicated in other wallets
+
     async def hack_populate_secret_key_for_puzzle_hash(self, puzzle_hash: bytes32) -> G1Element:
         maybe = await self.wallet_state_manager.get_keys(puzzle_hash)
         if maybe is None:
@@ -196,10 +199,10 @@ class Wallet:
 
     def make_solution(
         self,
-        primaries: Optional[List[Dict[str, Any]]] = None,
+        primaries: List[AmountWithPuzzlehash],
         min_time=0,
         me=None,
-        coin_announcements: Optional[Set[bytes32]] = None,
+        coin_announcements: Optional[Set[bytes]] = None,
         coin_announcements_to_assert: Optional[Set[bytes32]] = None,
         puzzle_announcements: Optional[Set[bytes32]] = None,
         puzzle_announcements_to_assert: Optional[Set[bytes32]] = None,
@@ -207,9 +210,15 @@ class Wallet:
     ) -> Program:
         assert fee >= 0
         condition_list = []
-        if primaries:
+        if len(primaries) > 0:
             for primary in primaries:
-                condition_list.append(make_create_coin_condition(primary["puzzlehash"], primary["amount"]))
+                if "memos" in primary:
+                    memos: Optional[List[bytes]] = primary["memos"]
+                    if memos is not None and len(memos) == 0:
+                        memos = None
+                else:
+                    memos = None
+                condition_list.append(make_create_coin_condition(primary["puzzlehash"], primary["amount"], memos))
         if min_time > 0:
             condition_list.append(make_assert_absolute_seconds_exceeds_condition(min_time))
         if me:
@@ -230,6 +239,11 @@ class Wallet:
                 condition_list.append(make_assert_puzzle_announcement(announcement_hash))
         return solution_for_conditions(condition_list)
 
+    def add_condition_to_solution(self, condition: Program, solution: Program) -> Program:
+        python_program = solution.as_python()
+        python_program[1].append(condition)
+        return Program.to(python_program)
+
     async def select_coins(self, amount, exclude: List[Coin] = None) -> Set[Coin]:
         """
         Returns a set of coins that can be used for generating a new transaction.
@@ -291,16 +305,19 @@ class Wallet:
         fee: uint64 = uint64(0),
         origin_id: bytes32 = None,
         coins: Set[Coin] = None,
-        primaries_input: Optional[List[Dict[str, Any]]] = None,
+        primaries_input: Optional[List[AmountWithPuzzlehash]] = None,
         ignore_max_send_amount: bool = False,
-        announcements_to_consume: Set[Announcement] = None,
+        coin_announcements_to_consume: Set[Announcement] = None,
+        puzzle_announcements_to_consume: Set[Announcement] = None,
+        memos: Optional[List[bytes]] = None,
+        negative_change_allowed: bool = False,
     ) -> List[CoinSpend]:
         """
         Generates a unsigned transaction in form of List(Puzzle, Solutions)
         Note: this must be called under a wallet state manager lock
         """
         if primaries_input is None:
-            primaries: Optional[List[Dict]] = None
+            primaries: Optional[List[AmountWithPuzzlehash]] = None
             total_amount = amount + fee
         else:
             primaries = primaries_input.copy()
@@ -317,12 +334,24 @@ class Wallet:
         if coins is None:
             coins = await self.select_coins(total_amount)
         assert len(coins) > 0
-
         self.log.info(f"coins is not None {coins}")
         spend_value = sum([coin.amount for coin in coins])
+
         change = spend_value - total_amount
+        if negative_change_allowed:
+            change = max(0, change)
+
         assert change >= 0
 
+        if coin_announcements_to_consume is not None:
+            coin_announcements_bytes: Optional[Set[bytes32]] = {a.name() for a in coin_announcements_to_consume}
+        else:
+            coin_announcements_bytes = None
+        if puzzle_announcements_to_consume is not None:
+            puzzle_announcements_bytes: Optional[Set[bytes32]] = {a.name() for a in puzzle_announcements_to_consume}
+        else:
+            puzzle_announcements_bytes = None
+
         spends: List[CoinSpend] = []
         primary_announcement_hash: Optional[bytes32] = None
 
@@ -331,41 +360,60 @@ class Wallet:
             all_primaries_list = [(p["puzzlehash"], p["amount"]) for p in primaries] + [(newpuzzlehash, amount)]
             if len(set(all_primaries_list)) != len(all_primaries_list):
                 raise ValueError("Cannot create two identical coins")
-
+        if memos is None:
+            memos = []
+        assert memos is not None
         for coin in coins:
-            self.log.info(f"coin from coins {coin}")
-            puzzle: Program = await self.puzzle_for_puzzle_hash(coin.puzzle_hash)
-
             # Only one coin creates outputs
-            if primary_announcement_hash is None and origin_id in (None, coin.name()):
+            if origin_id in (None, coin.name()):
+                origin_id = coin.name()
                 if primaries is None:
-                    primaries = [{"puzzlehash": newpuzzlehash, "amount": amount}]
+                    if amount > 0:
+                        primaries = [{"puzzlehash": newpuzzlehash, "amount": uint64(amount), "memos": memos}]
+                    else:
+                        primaries = []
                 else:
-                    primaries.append({"puzzlehash": newpuzzlehash, "amount": amount})
+                    primaries.append({"puzzlehash": newpuzzlehash, "amount": uint64(amount), "memos": memos})
                 if change > 0:
                     change_puzzle_hash: bytes32 = await self.get_new_puzzlehash()
-                    primaries.append({"puzzlehash": change_puzzle_hash, "amount": change})
+                    primaries.append({"puzzlehash": change_puzzle_hash, "amount": uint64(change), "memos": []})
                 message_list: List[bytes32] = [c.name() for c in coins]
                 for primary in primaries:
                     message_list.append(Coin(coin.name(), primary["puzzlehash"], primary["amount"]).name())
                 message: bytes32 = std_hash(b"".join(message_list))
+                puzzle: Program = await self.puzzle_for_puzzle_hash(coin.puzzle_hash)
                 solution: Program = self.make_solution(
                     primaries=primaries,
                     fee=fee,
                     coin_announcements={message},
-                    coin_announcements_to_assert=announcements_to_consume,
+                    coin_announcements_to_assert=coin_announcements_bytes,
+                    puzzle_announcements_to_assert=puzzle_announcements_bytes,
                 )
                 primary_announcement_hash = Announcement(coin.name(), message).name()
-            else:
-                solution = self.make_solution(coin_announcements_to_assert={primary_announcement_hash})
 
+                spends.append(
+                    CoinSpend(
+                        coin, SerializedProgram.from_bytes(bytes(puzzle)), SerializedProgram.from_bytes(bytes(solution))
+                    )
+                )
+                break
+        else:
+            raise ValueError("origin_id is not in the set of selected coins")
+
+        # Process the non-origin coins now that we have the primary announcement hash
+        for coin in coins:
+            if coin.name() == origin_id:
+                continue
+
+            puzzle = await self.puzzle_for_puzzle_hash(coin.puzzle_hash)
+            solution = self.make_solution(coin_announcements_to_assert={primary_announcement_hash}, primaries=[])
             spends.append(
                 CoinSpend(
                     coin, SerializedProgram.from_bytes(bytes(puzzle)), SerializedProgram.from_bytes(bytes(solution))
                 )
             )
 
-        self.log.info(f"Spends is {spends}")
+        self.log.debug(f"Spends is {spends}")
         return spends
 
     async def sign_transaction(self, coin_spends: List[CoinSpend]) -> SpendBundle:
@@ -383,13 +431,17 @@ class Wallet:
         fee: uint64 = uint64(0),
         origin_id: bytes32 = None,
         coins: Set[Coin] = None,
-        primaries: Optional[List[Dict[str, bytes32]]] = None,
+        primaries: Optional[List[AmountWithPuzzlehash]] = None,
         ignore_max_send_amount: bool = False,
-        announcements_to_consume: Set[Announcement] = None,
+        coin_announcements_to_consume: Set[Announcement] = None,
+        puzzle_announcements_to_consume: Set[Announcement] = None,
+        memos: Optional[List[bytes]] = None,
+        negative_change_allowed: bool = False,
     ) -> TransactionRecord:
         """
         Use this to generate transaction.
         Note: this must be called under a wallet state manager lock
+        The first output is (amount, puzzle_hash, memos), and the rest of the outputs are in primaries.
         """
         if primaries is None:
             non_change_amount = amount
@@ -397,7 +449,17 @@ class Wallet:
             non_change_amount = uint64(amount + sum(p["amount"] for p in primaries))
 
         transaction = await self._generate_unsigned_transaction(
-            amount, puzzle_hash, fee, origin_id, coins, primaries, ignore_max_send_amount, announcements_to_consume
+            amount,
+            puzzle_hash,
+            fee,
+            origin_id,
+            coins,
+            primaries,
+            ignore_max_send_amount,
+            coin_announcements_to_consume,
+            puzzle_announcements_to_consume,
+            memos,
+            negative_change_allowed,
         )
         assert len(transaction) > 0
 
@@ -413,7 +475,13 @@ class Wallet:
         now = uint64(int(time.time()))
         add_list: List[Coin] = list(spend_bundle.additions())
         rem_list: List[Coin] = list(spend_bundle.removals())
-        assert sum(a.amount for a in add_list) + fee == sum(r.amount for r in rem_list)
+
+        output_amount = sum(a.amount for a in add_list) + fee
+        input_amount = sum(r.amount for r in rem_list)
+        if negative_change_allowed:
+            assert output_amount >= input_amount
+        else:
+            assert output_amount == input_amount
 
         return TransactionRecord(
             confirmed_at_height=uint32(0),
@@ -431,13 +499,15 @@ class Wallet:
             trade_id=None,
             type=uint32(TransactionType.OUTGOING_TX.value),
             name=spend_bundle.name(),
+            memos=list(compute_memos(spend_bundle).items()),
         )
 
     async def push_transaction(self, tx: TransactionRecord) -> None:
         """Use this API to send transactions."""
         await self.wallet_state_manager.add_pending_transaction(tx)
+        await self.wallet_state_manager.wallet_node.update_ui()
 
-    # This is to be aggregated together with a coloured coin offer to ensure that the trade happens
+    # This is to be aggregated together with a CAT offer to ensure that the trade happens
     async def create_spend_bundle_relative_chia(self, chia_amount: int, exclude: List[Coin]) -> SpendBundle:
         list_of_solutions = []
         utxos = None
@@ -461,7 +531,9 @@ class Wallet:
             puzzle = await self.puzzle_for_puzzle_hash(coin.puzzle_hash)
             if output_created is None:
                 newpuzhash = await self.get_new_puzzlehash()
-                primaries = [{"puzzlehash": newpuzhash, "amount": chia_amount}]
+                primaries: List[AmountWithPuzzlehash] = [
+                    {"puzzlehash": newpuzhash, "amount": uint64(chia_amount), "memos": []}
+                ]
                 solution = self.make_solution(primaries=primaries)
                 output_created = coin
             list_of_solutions.append(CoinSpend(coin, puzzle, solution))
