@@ -1,5 +1,30 @@
 #!/bin/bash
 
+set -o errexit
+
+USAGE_TEXT="\
+Usage: $0 [-d]
+
+  -n                          do not install Python development package, Python.h etc
+  -h                          display this help and exit
+"
+
+usage() {
+  echo "${USAGE_TEXT}"
+}
+
+INSTALL_PYTHON_DEV=1
+
+while getopts nh flag
+do
+  case "${flag}" in
+    # development
+    n) INSTALL_PYTHON_DEV=;;
+    h) usage; exit 0;;
+    *) echo; usage; exit 1;;
+  esac
+done
+
 if [ -z "$VIRTUAL_ENV" ]; then
   echo "This requires the chia python virtual environment."
   echo "Execute '. ./activate' before running."
@@ -11,6 +36,12 @@ echo "Timelord requires CMake 3.14+ to compile vdf_client."
 PYTHON_VERSION=$(python -c 'import sys; print(f"python{sys.version_info.major}.{sys.version_info.minor}")')
 echo "Python version: $PYTHON_VERSION"
 
+if [ "$INSTALL_PYTHON_DEV" ]; then
+  PYTHON_DEV_DEPENDENCY=lib"$PYTHON_VERSION"-dev
+else
+  PYTHON_DEV_DEPENDENCY=
+fi
+
 export BUILD_VDF_BENCH=Y # Installs the useful vdf_bench test of CPU squaring speed
 THE_PATH=$(python -c 'import pkg_resources; print( pkg_resources.get_distribution("chiavdf").location)' 2>/dev/null)/vdf_client
 CHIAVDF_VERSION=$(python -c 'from setup import dependencies; t = [_ for _ in dependencies if _.startswith("chiavdf")][0]; print(t)')
@@ -18,11 +49,12 @@ CHIAVDF_VERSION=$(python -c 'from setup import dependencies; t = [_ for _ in dep
 ubuntu_cmake_install() {
 	UBUNTU_PRE_2004=$(python -c 'import subprocess; process = subprocess.run(["lsb_release", "-rs"], stdout=subprocess.PIPE); print(float(process.stdout) < float(20.04))')
 	if [ "$UBUNTU_PRE_2004" = "True" ]; then
-		echo "Ubuntu version is pre 20.04LTS - installing CMake with snap."
-		sudo apt-get install snap -y
+		echo "Installing CMake with snap."
+		sudo apt-get install snapd -y
 		sudo apt-get remove --purge cmake -y
 		hash -r
 		sudo snap install cmake --classic
+		. /etc/profile
 	else
 		echo "Ubuntu 20.04LTS and newer support CMake 3.16+"
 		sudo apt-get install cmake -y
@@ -56,26 +88,28 @@ if [ -e "$THE_PATH" ]; then
 	echo "vdf_client already exists, no action taken"
 else
 	if [ -e venv/bin/python ] && test $UBUNTU_DEBIAN; then
-		echo "Installing chiavdf from source on Ubuntu/Debian"
+		echo "Installing chiavdf dependencies on Ubuntu/Debian"
 		# If Ubuntu version is older than 20.04LTS then upgrade CMake
 		ubuntu_cmake_install
 		# Install remaining needed development tools - assumes venv and prior run of install.sh
-		echo apt-get install libgmp-dev libboost-python-dev lib"$PYTHON_VERSION"-dev libboost-system-dev build-essential -y
-		sudo apt-get install libgmp-dev libboost-python-dev lib"$PYTHON_VERSION"-dev libboost-system-dev build-essential -y
+		echo "apt-get install libgmp-dev libboost-python-dev $PYTHON_DEV_DEPENDENCY libboost-system-dev build-essential -y"
+		sudo apt-get install libgmp-dev libboost-python-dev "$PYTHON_DEV_DEPENDENCY" libboost-system-dev build-essential -y
+		echo "Installing chiavdf from source on Ubuntu/Debian"
 		echo venv/bin/python -m pip install --force --no-binary chiavdf "$CHIAVDF_VERSION"
 		venv/bin/python -m pip install --force --no-binary chiavdf "$CHIAVDF_VERSION"
 		symlink_vdf_bench "$PYTHON_VERSION"
 	elif [ -e venv/bin/python ] && test $RHEL_BASED; then
-		echo "Installing chiavdf from source on RedHat/CentOS/Fedora"
+		echo "Installing chiavdf dependencies on RedHat/CentOS/Fedora"
 		# Install remaining needed development tools - assumes venv and prior run of install.sh
-		echo yum install gcc gcc-c++ gmp-devel python3-devel libtool make autoconf automake openssl-devel libevent-devel boost-devel python3 -y
-		sudo yum install gcc gcc-c++ gmp-devel python3-devel libtool make autoconf automake openssl-devel libevent-devel boost-devel python3 -y
+		echo "yum install gcc gcc-c++ gmp-devel $PYTHON_DEV_DEPENDENCY libtool make autoconf automake openssl-devel libevent-devel boost-devel python3 cmake -y"
+		sudo yum install gcc gcc-c++ gmp-devel "$PYTHON_DEV_DEPENDENCY" libtool make autoconf automake openssl-devel libevent-devel boost-devel python3 cmake -y
+		echo "Installing chiavdf from source on RedHat/CentOS/Fedora"
 		echo venv/bin/python -m pip install --force --no-binary chiavdf "$CHIAVDF_VERSION"
 		venv/bin/python -m pip install --force --no-binary chiavdf "$CHIAVDF_VERSION"
 		symlink_vdf_bench "$PYTHON_VERSION"
-	elif [ -e venv/bin/python ] && test $MACOS && [ "$(brew info boost | grep -c 'Not installed')" -eq 1 ]; then
-		echo "Installing chiavdf requirement boost for MacOS."
-		brew install boost
+	elif [ -e venv/bin/python ] && test $MACOS; then
+		echo "Installing chiavdf dependencies for MacOS."
+		brew install boost cmake gmp
 		echo "Installing chiavdf from source."
 		# User needs to provide required packages
 		echo venv/bin/python -m pip install --force --no-binary chiavdf "$CHIAVDF_VERSION"
