@@ -11,7 +11,7 @@ import time
 from argparse import Namespace
 from dataclasses import replace
 from pathlib import Path
-from typing import Callable, Dict, List, Optional, Tuple, Any
+from typing import Callable, Dict, List, Optional, Tuple, Any, Union
 
 from blspy import AugSchemeMPL, G1Element, G2Element, PrivateKey
 from chiabip158 import PyBIP158
@@ -27,6 +27,7 @@ from chia.util.errors import Err
 from chia.full_node.generator import setup_generator_args
 from chia.full_node.mempool_check_conditions import GENERATOR_MOD
 from chia.plotting.create_plots import create_plots, PlotKeys
+from chia.plotting.util import add_plot_directory
 from chia.consensus.block_creation import unfinished_block_to_full_block
 from chia.consensus.block_record import BlockRecord
 from chia.consensus.block_rewards import calculate_base_farmer_reward, calculate_pool_reward
@@ -74,7 +75,8 @@ from chia.types.spend_bundle import SpendBundle
 from chia.types.unfinished_block import UnfinishedBlock
 from chia.util.bech32m import encode_puzzle_hash
 from chia.util.block_cache import BlockCache
-from chia.util.config import load_config, save_config
+from chia.util.config import load_config, lock_config, save_config
+from chia.util.default_root import DEFAULT_ROOT_PATH
 from chia.util.hash import std_hash
 from chia.util.ints import uint8, uint16, uint32, uint64, uint128
 from chia.util.keychain import Keychain, bytes_to_mnemonic
@@ -82,9 +84,10 @@ from chia.util.merkle_set import MerkleSet
 from chia.util.prev_transaction_block import get_prev_transaction_block
 from chia.util.path import mkdir
 from chia.util.vdf_prover import get_vdf_info_and_proof
-from tests.time_out_assert import time_out_assert
+from tests.time_out_assert import time_out_assert_custom_interval
 from tests.wallet_tools import WalletTool
 from tests.util.socket import find_available_listen_port
+from tests.util.ssl_certs import get_next_nodes_certs_and_keys, get_next_private_ca_cert_and_key
 from chia.wallet.derive_keys import (
     master_sk_to_farmer_sk,
     master_sk_to_local_sk,
@@ -96,7 +99,7 @@ test_constants = DEFAULT_CONSTANTS.replace(
     **{
         "MIN_PLOT_SIZE": 18,
         "MIN_BLOCKS_PER_CHALLENGE_BLOCK": 12,
-        "DIFFICULTY_STARTING": 2 ** 12,
+        "DIFFICULTY_STARTING": 2 ** 10,
         "DISCRIMINANT_SIZE_BITS": 16,
         "SUB_EPOCH_BLOCKS": 170,
         "WEIGHT_PROOF_THRESHOLD": 2,
@@ -143,7 +146,11 @@ class BlockTools:
         self.local_keychain = keychain
 
         create_default_chia_config(root_path)
-        create_all_ssl(root_path)
+        create_all_ssl(
+            root_path,
+            private_ca_crt_and_key=get_next_private_ca_cert_and_key(),
+            node_certs_and_keys=get_next_nodes_certs_and_keys(),
+        )
 
         self.local_sk_cache: Dict[bytes32, Tuple[PrivateKey, Any]] = {}
         self._config = load_config(self.root_path, "config.yaml")
@@ -155,7 +162,8 @@ class BlockTools:
         # some tests start the daemon, make sure it's on a free port
         self._config["daemon_port"] = find_available_listen_port("BlockTools daemon")
 
-        save_config(self.root_path, "config.yaml", self._config)
+        with lock_config(self.root_path, "config.yaml"):
+            save_config(self.root_path, "config.yaml", self._config)
         overrides = self._config["network_overrides"]["constants"][self._config["selected_network"]]
         updated_constants = constants.replace_str_to_bytes(**overrides)
         if const_dict is not None:
@@ -233,9 +241,14 @@ class BlockTools:
         overrides = self._config["network_overrides"]["constants"][self._config["selected_network"]]
         updated_constants = self.constants.replace_str_to_bytes(**overrides)
         self.constants = updated_constants
-        save_config(self.root_path, "config.yaml", self._config)
+        with lock_config(self.root_path, "config.yaml"):
+            save_config(self.root_path, "config.yaml", self._config)
+
+    def add_plot_directory(self, path: Path) -> None:
+        self._config = add_plot_directory(self.root_path, str(path))
 
     async def setup_plots(self):
+        self.add_plot_directory(self.plot_dir)
         assert self.created_plots == 0
         # OG Plots
         for i in range(15):
@@ -248,7 +261,7 @@ class BlockTools:
             await self.new_plot(
                 path=self.plot_dir / "not_in_keychain",
                 plot_keys=PlotKeys(G1Element(), G1Element(), None),
-                exclude_final_dir=True,
+                exclude_plots=True,
             )
 
         await self.refresh_plots()
@@ -257,21 +270,24 @@ class BlockTools:
         self,
         pool_contract_puzzle_hash: Optional[bytes32] = None,
         path: Path = None,
+        tmp_dir: Path = None,
         plot_keys: Optional[PlotKeys] = None,
-        exclude_final_dir: bool = False,
+        exclude_plots: bool = False,
     ) -> Optional[bytes32]:
         final_dir = self.plot_dir
         if path is not None:
             final_dir = path
             mkdir(final_dir)
+        if tmp_dir is None:
+            tmp_dir = self.temp_dir
         args = Namespace()
         # Can't go much lower than 20, since plots start having no solutions and more buggy
-        args.size = 22
+        args.size = 20
         # Uses many plots for testing, in order to guarantee proofs of space at every height
         args.num = 1
         args.buffer = 100
-        args.tmp_dir = self.temp_dir
-        args.tmp2_dir = self.temp_dir
+        args.tmp_dir = tmp_dir
+        args.tmp2_dir = tmp_dir
         args.final_dir = final_dir
         args.plotid = None
         args.memo = None
@@ -281,7 +297,6 @@ class BlockTools:
         args.nobitfield = False
         args.exclude_final_dir = False
         args.list_duplicates = False
-        args.exclude_final_dir = exclude_final_dir
         try:
             if plot_keys is None:
                 pool_pk: Optional[G1Element] = None
@@ -296,14 +311,13 @@ class BlockTools:
             created, existed = await create_plots(
                 args,
                 plot_keys,
-                self.root_path,
                 use_datetime=False,
                 test_private_keys=[AugSchemeMPL.key_gen(std_hash(self.created_plots.to_bytes(2, "big")))],
             )
             self.created_plots += 1
 
             plot_id_new: Optional[bytes32] = None
-            path_new: Path = Path()
+            path_new: Optional[Path] = None
 
             if len(created):
                 assert len(existed) == 0
@@ -312,17 +326,11 @@ class BlockTools:
             if len(existed):
                 assert len(created) == 0
                 plot_id_new, path_new = list(existed.items())[0]
+            assert plot_id_new is not None
+            assert path_new is not None
 
-            if not exclude_final_dir:
-                # TODO: address hint error and remove ignore
-                #       error: Invalid index type "Optional[bytes32]" for "Dict[bytes32, Path]"; expected type "bytes32"
-                #       [index]
-                self.expected_plots[plot_id_new] = path_new  # type: ignore[index]
-
-            # create_plots() updates plot_directories. Ensure we refresh our config to reflect the updated value
-            self._config["harvester"]["plot_directories"] = load_config(self.root_path, "config.yaml", "harvester")[
-                "plot_directories"
-            ]
+            if not exclude_plots:
+                self.expected_plots[plot_id_new] = path_new
 
             return plot_id_new
 
@@ -336,8 +344,8 @@ class BlockTools:
         )  # Make sure we have at least some batches + a remainder
         self.plot_manager.trigger_refresh()
         assert self.plot_manager.needs_refresh()
-        self.plot_manager.start_refreshing()
-        await time_out_assert(10, self.plot_manager.needs_refresh, value=False)
+        self.plot_manager.start_refreshing(sleep_interval_ms=1)
+        await time_out_assert_custom_interval(10, 0.001, self.plot_manager.needs_refresh, value=False)
         self.plot_manager.stop_refreshing()
         assert not self.plot_manager.needs_refresh()
 
@@ -351,7 +359,7 @@ class BlockTools:
     def config(self) -> Dict:
         return copy.deepcopy(self._config)
 
-    def get_daemon_ssl_context(self) -> Optional[ssl.SSLContext]:
+    def get_daemon_ssl_context(self) -> ssl.SSLContext:
         crt_path = self.root_path / self.config["daemon_ssl"]["private_crt"]
         key_path = self.root_path / self.config["daemon_ssl"]["private_key"]
         ca_cert_path = self.root_path / self.config["private_ssl_ca"]["crt"]
@@ -411,6 +419,7 @@ class BlockTools:
         self,
         num_blocks: int,
         block_list_input: List[FullBlock] = None,
+        *,
         farmer_reward_puzzle_hash: Optional[bytes32] = None,
         pool_reward_puzzle_hash: Optional[bytes32] = None,
         transaction_data: Optional[SpendBundle] = None,
@@ -419,12 +428,13 @@ class BlockTools:
         force_overflow: bool = False,
         skip_slots: int = 0,  # Force at least this number of empty slots before the first SB
         guarantee_transaction_block: bool = False,  # Force that this block must be a tx block
+        keep_going_until_tx_block: bool = False,  # keep making new blocks until we find a tx block
         normalized_to_identity_cc_eos: bool = False,
         normalized_to_identity_icc_eos: bool = False,
         normalized_to_identity_cc_sp: bool = False,
         normalized_to_identity_cc_ip: bool = False,
         current_time: bool = False,
-        previous_generator: CompressorArg = None,
+        previous_generator: Optional[Union[CompressorArg, List[uint32]]] = None,
         genesis_timestamp: Optional[uint64] = None,
         force_plot_id: Optional[bytes32] = None,
     ) -> List[FullBlock]:
@@ -445,12 +455,9 @@ class BlockTools:
             if force_plot_id is not None:
                 raise ValueError("Cannot specify plot_id for genesis block")
             initial_block_list_len = 0
-            # TODO: address hint error and remove ignore
-            #       error: Argument 2 to "create_genesis_block" of "BlockTools" has incompatible type "bytes"; expected
-            #       "bytes32"  [arg-type]
             genesis = self.create_genesis_block(
                 constants,
-                seed,  # type: ignore[arg-type]
+                seed,
                 force_overflow=force_overflow,
                 skip_slots=skip_slots,
                 timestamp=(uint64(int(time.time())) if genesis_timestamp is None else genesis_timestamp),
@@ -466,14 +473,13 @@ class BlockTools:
         if num_blocks == 0:
             return block_list
 
+        blocks: Dict[bytes32, BlockRecord]
         height_to_hash, difficulty, blocks = load_block_list(block_list, constants)
 
         latest_block: BlockRecord = blocks[block_list[-1].header_hash]
         curr = latest_block
         while not curr.is_transaction_block:
-            # TODO: address hint error and remove ignore
-            #       error: Invalid index type "bytes32" for "Dict[uint32, BlockRecord]"; expected type "uint32"  [index]
-            curr = blocks[curr.prev_hash]  # type: ignore[index]
+            curr = blocks[curr.prev_hash]
         start_timestamp = curr.timestamp
         start_height = curr.height
 
@@ -481,9 +487,7 @@ class BlockTools:
         blocks_added_this_sub_slot = 1
 
         while not curr.first_in_sub_slot:
-            # TODO: address hint error and remove ignore
-            #       error: Invalid index type "bytes32" for "Dict[uint32, BlockRecord]"; expected type "uint32"  [index]
-            curr = blocks[curr.prev_hash]  # type: ignore[index]
+            curr = blocks[curr.prev_hash]
             blocks_added_this_sub_slot += 1
 
         finished_sub_slots_at_sp: List[EndOfSubSlotBundle] = []  # Sub-slots since last block, up to signage point
@@ -516,10 +520,7 @@ class BlockTools:
                     ):
                         if curr.height == 0:
                             break
-                        # TODO: address hint error and remove ignore
-                        #       error: Invalid index type "bytes32" for "Dict[uint32, BlockRecord]"; expected type
-                        #       "uint32"  [index]
-                        curr = blocks[curr.prev_hash]  # type: ignore[index]
+                        curr = blocks[curr.prev_hash]
                     if curr.total_iters > sub_slot_start_total_iters:
                         finished_sub_slots_at_sp = []
 
@@ -528,12 +529,9 @@ class BlockTools:
                             # Ignore this signage_point because it's in the past
                             continue
 
-                    # TODO: address hint error and remove ignore
-                    #       error: Argument 1 to "BlockCache" has incompatible type "Dict[uint32, BlockRecord]";
-                    #       expected "Dict[bytes32, BlockRecord]"  [arg-type]
                     signage_point: SignagePoint = get_signage_point(
                         constants,
-                        BlockCache(blocks),  # type: ignore[arg-type]
+                        BlockCache(blocks),
                         latest_block,
                         sub_slot_start_total_iters,
                         uint8(signage_point_index),
@@ -570,7 +568,8 @@ class BlockTools:
                         removals = None
                         if transaction_data_included:
                             transaction_data = None
-                        if transaction_data is not None and not transaction_data_included:
+                            previous_generator = None
+                        if transaction_data is not None:
                             additions = transaction_data.additions()
                             removals = transaction_data.removals()
                         assert start_timestamp is not None
@@ -586,22 +585,22 @@ class BlockTools:
                             else:
                                 pool_target = PoolTarget(self.pool_ph, uint32(0))
 
+                        block_generator: Optional[BlockGenerator]
                         if transaction_data is not None:
-                            if previous_generator is not None:
-                                block_generator: Optional[BlockGenerator] = best_solution_generator_from_template(
+                            if type(previous_generator) is CompressorArg:
+                                block_generator = best_solution_generator_from_template(
                                     previous_generator, transaction_data
                                 )
                             else:
                                 block_generator = simple_solution_generator(transaction_data)
+                                if type(previous_generator) is list:
+                                    block_generator = BlockGenerator(block_generator.program, [], previous_generator)
 
                             aggregate_signature = transaction_data.aggregated_signature
                         else:
                             block_generator = None
                             aggregate_signature = G2Element()
 
-                        # TODO: address hint error and remove ignore
-                        #       error: Argument 27 to "get_full_block_and_block_record" has incompatible type "bool";
-                        #       expected "Optional[bytes32]"  [arg-type]
                         full_block, block_record = get_full_block_and_block_record(
                             constants,
                             blocks,
@@ -629,11 +628,13 @@ class BlockTools:
                             signage_point,
                             latest_block,
                             seed,
-                            normalized_to_identity_cc_ip,  # type: ignore[arg-type]
+                            normalized_to_identity_cc_ip=normalized_to_identity_cc_ip,
                             current_time=current_time,
                         )
                         if block_record.is_transaction_block:
                             transaction_data_included = True
+                            previous_generator = None
+                            keep_going_until_tx_block = False
                         else:
                             if guarantee_transaction_block:
                                 continue
@@ -655,7 +656,7 @@ class BlockTools:
                         latest_block = blocks[full_block.header_hash]
                         finished_sub_slots_at_ip = []
                         num_blocks -= 1
-                        if num_blocks == 0:
+                        if num_blocks <= 0 and not keep_going_until_tx_block:
                             return block_list
 
             # Finish the end of sub-slot and try again next sub-slot
@@ -686,15 +687,12 @@ class BlockTools:
             eos_deficit: uint8 = (
                 latest_block.deficit if latest_block.deficit > 0 else constants.MIN_BLOCKS_PER_CHALLENGE_BLOCK
             )
-            # TODO: address hint error and remove ignore
-            #       error: Argument 5 to "get_icc" has incompatible type "Dict[uint32, BlockRecord]"; expected
-            #       "Dict[bytes32, BlockRecord]"  [arg-type]
             icc_eos_vdf, icc_ip_proof = get_icc(
                 constants,
                 uint128(sub_slot_start_total_iters + sub_slot_iters),
                 finished_sub_slots_at_ip,
                 latest_block,
-                blocks,  # type: ignore[arg-type]
+                blocks,
                 sub_slot_start_total_iters,
                 eos_deficit,
             )
@@ -712,14 +710,9 @@ class BlockTools:
             if pending_ses:
                 sub_epoch_summary: Optional[SubEpochSummary] = None
             else:
-                # TODO: address hint error and remove ignore
-                #       error: Argument 1 to "BlockCache" has incompatible type "Dict[uint32, BlockRecord]"; expected
-                #       "Dict[bytes32, BlockRecord]"  [arg-type]
-                #       error: Argument 2 to "BlockCache" has incompatible type "Dict[uint32, bytes32]"; expected
-                #       "Optional[Dict[bytes32, HeaderBlock]]"  [arg-type]
                 sub_epoch_summary = next_sub_epoch_summary(
                     constants,
-                    BlockCache(blocks, height_to_hash),  # type: ignore[arg-type]
+                    BlockCache(blocks, height_to_hash=height_to_hash),
                     latest_block.required_iters,
                     block_list[-1],
                     False,
@@ -744,10 +737,7 @@ class BlockTools:
                     # This means there are blocks in this sub-slot
                     curr = latest_block
                     while not curr.is_challenge_block(constants) and not curr.first_in_sub_slot:
-                        # TODO: address hint error and remove ignore
-                        #       error: Invalid index type "bytes32" for "Dict[uint32, BlockRecord]"; expected type
-                        #       "uint32"  [index]
-                        curr = blocks[curr.prev_hash]  # type: ignore[index]
+                        curr = blocks[curr.prev_hash]
                     if curr.is_challenge_block(constants):
                         icc_eos_iters = uint64(sub_slot_start_total_iters + sub_slot_iters - curr.total_iters)
                     else:
@@ -805,7 +795,7 @@ class BlockTools:
             removals = None
             if transaction_data_included:
                 transaction_data = None
-            if transaction_data is not None and not transaction_data_included:
+            if transaction_data is not None:
                 additions = transaction_data.additions()
                 removals = transaction_data.removals()
             sub_slots_finished += 1
@@ -822,12 +812,9 @@ class BlockTools:
                     constants.NUM_SPS_SUB_SLOT,
                 ):
                     # note that we are passing in the finished slots which include the last slot
-                    # TODO: address hint error and remove ignore
-                    #       error: Argument 1 to "BlockCache" has incompatible type "Dict[uint32, BlockRecord]";
-                    #       expected "Dict[bytes32, BlockRecord]"  [arg-type]
                     signage_point = get_signage_point(
                         constants,
-                        BlockCache(blocks),  # type: ignore[arg-type]
+                        BlockCache(blocks),
                         latest_block_eos,
                         sub_slot_start_total_iters,
                         uint8(signage_point_index),
@@ -869,12 +856,14 @@ class BlockTools:
                             else:
                                 pool_target = PoolTarget(self.pool_ph, uint32(0))
                         if transaction_data is not None:
-                            if previous_generator is not None:
+                            if previous_generator is not None and type(previous_generator) is CompressorArg:
                                 block_generator = best_solution_generator_from_template(
                                     previous_generator, transaction_data
                                 )
                             else:
                                 block_generator = simple_solution_generator(transaction_data)
+                                if type(previous_generator) is list:
+                                    block_generator = BlockGenerator(block_generator.program, [], previous_generator)
                             aggregate_signature = transaction_data.aggregated_signature
                         else:
                             block_generator = None
@@ -914,6 +903,8 @@ class BlockTools:
 
                         if block_record.is_transaction_block:
                             transaction_data_included = True
+                            previous_generator = None
+                            keep_going_until_tx_block = False
                         elif guarantee_transaction_block:
                             continue
                         if pending_ses:
@@ -930,7 +921,7 @@ class BlockTools:
                         blocks_added_this_sub_slot += 1
                         log.info(f"Created block {block_record.height } ov=True, iters " f"{block_record.total_iters}")
                         num_blocks -= 1
-                        if num_blocks == 0:
+                        if num_blocks <= 0 and not keep_going_until_tx_block:
                             return block_list
 
                         blocks[full_block.header_hash] = block_record
@@ -949,13 +940,10 @@ class BlockTools:
                 sub_slot_iters = new_sub_slot_iters
                 difficulty = new_difficulty
 
-    # TODO: address hint error and remove ignore
-    #       error: Incompatible default for argument "seed" (default has type "bytes", argument has type "bytes32")
-    #       [assignment]
     def create_genesis_block(
         self,
         constants: ConsensusConstants,
-        seed: bytes32 = b"",  # type: ignore[assignment]
+        seed: bytes = b"",
         timestamp: Optional[uint64] = None,
         force_overflow: bool = False,
         skip_slots: int = 0,
@@ -1348,20 +1336,16 @@ def finish_block(
 
 def get_challenges(
     constants: ConsensusConstants,
-    blocks: Dict[uint32, BlockRecord],
+    blocks: Dict[bytes32, BlockRecord],
     finished_sub_slots: List[EndOfSubSlotBundle],
     prev_header_hash: Optional[bytes32],
 ) -> Tuple[bytes32, bytes32]:
     if len(finished_sub_slots) == 0:
         if prev_header_hash is None:
             return constants.GENESIS_CHALLENGE, constants.GENESIS_CHALLENGE
-        # TODO: address hint error and remove ignore
-        #       error: Invalid index type "bytes32" for "Dict[uint32, BlockRecord]"; expected type "uint32"  [index]
-        curr: BlockRecord = blocks[prev_header_hash]  # type: ignore[index]
+        curr: BlockRecord = blocks[prev_header_hash]
         while not curr.first_in_sub_slot:
-            # TODO: address hint error and remove ignore
-            #       error: Invalid index type "bytes32" for "Dict[uint32, BlockRecord]"; expected type "uint32"  [index]
-            curr = blocks[curr.prev_hash]  # type: ignore[index]
+            curr = blocks[curr.prev_hash]
         assert curr.finished_challenge_slot_hashes is not None
         assert curr.finished_reward_slot_hashes is not None
         cc_challenge = curr.finished_challenge_slot_hashes[-1]
@@ -1373,7 +1357,12 @@ def get_challenges(
 
 
 def get_plot_dir() -> Path:
-    cache_path = Path(os.path.expanduser(os.getenv("CHIA_ROOT", "~/.chia/"))) / "test-plots"
+    cache_path = DEFAULT_ROOT_PATH.parent.joinpath("test-plots")
+
+    ci = os.environ.get("CI")
+    if ci is not None and not cache_path.exists():
+        raise Exception(f"Running in CI and expected path not found: {cache_path!r}")
+
     mkdir(cache_path)
     return cache_path
 
@@ -1384,10 +1373,10 @@ def get_plot_tmp_dir():
 
 def load_block_list(
     block_list: List[FullBlock], constants: ConsensusConstants
-) -> Tuple[Dict[uint32, bytes32], uint64, Dict[uint32, BlockRecord]]:
+) -> Tuple[Dict[uint32, bytes32], uint64, Dict[bytes32, BlockRecord]]:
     difficulty = 0
     height_to_hash: Dict[uint32, bytes32] = {}
-    blocks: Dict[uint32, BlockRecord] = {}
+    blocks: Dict[bytes32, BlockRecord] = {}
     for full_block in block_list:
         if full_block.height == 0:
             difficulty = uint64(constants.DIFFICULTY_STARTING)
@@ -1403,23 +1392,18 @@ def load_block_list(
         quality_str = full_block.reward_chain_block.proof_of_space.verify_and_get_quality_string(
             constants, challenge, sp_hash
         )
-        # TODO: address hint error and remove ignore
-        #       error: Argument 2 to "calculate_iterations_quality" has incompatible type "Optional[bytes32]"; expected
-        #       "bytes32"  [arg-type]
+        assert quality_str is not None
         required_iters: uint64 = calculate_iterations_quality(
             constants.DIFFICULTY_CONSTANT_FACTOR,
-            quality_str,  # type: ignore[arg-type]
+            quality_str,
             full_block.reward_chain_block.proof_of_space.size,
             uint64(difficulty),
             sp_hash,
         )
 
-        # TODO: address hint error and remove ignore
-        #       error: Argument 1 to "BlockCache" has incompatible type "Dict[uint32, BlockRecord]"; expected
-        #       "Dict[bytes32, BlockRecord]"  [arg-type]
         blocks[full_block.header_hash] = block_to_block_record(
             constants,
-            BlockCache(blocks),  # type: ignore[arg-type]
+            BlockCache(blocks),
             required_iters,
             full_block,
             None,
@@ -1487,7 +1471,7 @@ def get_icc(
 
 def get_full_block_and_block_record(
     constants: ConsensusConstants,
-    blocks: Dict[uint32, BlockRecord],
+    blocks: Dict[bytes32, BlockRecord],
     sub_slot_start_total_iters: uint128,
     signage_point_index: uint8,
     proof_of_space: ProofOfSpace,
@@ -1512,6 +1496,7 @@ def get_full_block_and_block_record(
     signage_point: SignagePoint,
     prev_block: BlockRecord,
     seed: bytes = b"",
+    *,
     overflow_cc_challenge: bytes32 = None,
     overflow_rc_challenge: bytes32 = None,
     normalized_to_identity_cc_ip: bool = False,
@@ -1527,11 +1512,6 @@ def get_full_block_and_block_record(
     sp_iters = calculate_sp_iters(constants, sub_slot_iters, signage_point_index)
     ip_iters = calculate_ip_iters(constants, sub_slot_iters, signage_point_index, required_iters)
 
-    # TODO: address hint error and remove ignore
-    #       error: Argument 1 to "BlockCache" has incompatible type "Dict[uint32, BlockRecord]"; expected
-    #       "Dict[bytes32, BlockRecord]"  [arg-type]
-    #       error: Argument 16 to "create_test_unfinished_block" has incompatible type "bytes"; expected "bytes32"
-    #       [arg-type]
     unfinished_block = create_test_unfinished_block(
         constants,
         sub_slot_start_total_iters,
@@ -1547,8 +1527,8 @@ def get_full_block_and_block_record(
         get_pool_signature,
         signage_point,
         timestamp,
-        BlockCache(blocks),  # type: ignore[arg-type]
-        seed,  # type: ignore[arg-type]
+        BlockCache(blocks),
+        seed,
         block_generator,
         aggregate_signature,
         additions,
@@ -1561,12 +1541,9 @@ def get_full_block_and_block_record(
         slot_cc_challenge = overflow_cc_challenge
         slot_rc_challenge = overflow_rc_challenge
 
-    # TODO: address hint error and remove ignore
-    #       error: Argument 2 to "finish_block" has incompatible type "Dict[uint32, BlockRecord]"; expected
-    #       "Dict[bytes32, BlockRecord]"  [arg-type]
     full_block, block_record = finish_block(
         constants,
-        blocks,  # type: ignore[arg-type]
+        blocks,
         height_to_hash,
         finished_sub_slots,
         sub_slot_start_total_iters,
@@ -1607,9 +1584,6 @@ def compute_cost_test(generator: BlockGenerator, cost_per_byte: int) -> Tuple[Op
         return uint16(Err.GENERATOR_RUNTIME_ERROR.value), uint64(0)
 
 
-# TODO: address hint error and remove ignore
-#       error: Incompatible default for argument "seed" (default has type "bytes", argument has type "bytes32")
-#       [assignment]
 def create_test_foliage(
     constants: ConsensusConstants,
     reward_block_unfinished: RewardChainBlockUnfinished,
@@ -1625,7 +1599,7 @@ def create_test_foliage(
     pool_target: PoolTarget,
     get_plot_signature: Callable[[bytes32, G1Element], G2Element],
     get_pool_signature: Callable[[PoolTarget, Optional[G1Element]], Optional[G2Element]],
-    seed: bytes32 = b"",  # type: ignore[assignment]
+    seed: bytes = b"",
 ) -> Tuple[Foliage, Optional[FoliageTransactionBlock], Optional[TransactionsInfo]]:
     """
     Creates a foliage for a given reward chain block. This may or may not be a tx block. In the case of a tx block,
@@ -1660,17 +1634,14 @@ def create_test_foliage(
 
     random.seed(seed)
     # Use the extension data to create different blocks based on header hash
-    # TODO: address hint error and remove ignore
-    #       error: Incompatible types in assignment (expression has type "bytes", variable has type "bytes32")
-    #       [assignment]
-    extension_data: bytes32 = random.randint(0, 100000000).to_bytes(32, "big")  # type: ignore[assignment]
+    extension_data: bytes32 = bytes32(random.randint(0, 100000000).to_bytes(32, "big"))
     if prev_block is None:
         height: uint32 = uint32(0)
     else:
         height = uint32(prev_block.height + 1)
 
     # Create filter
-    byte_array_tx: List[bytes32] = []
+    byte_array_tx: List[bytearray] = []
     tx_additions: List[Coin] = []
     tx_removals: List[bytes32] = []
 
@@ -1764,16 +1735,10 @@ def create_test_foliage(
         additions.extend(reward_claims_incorporated.copy())
         for coin in additions:
             tx_additions.append(coin)
-            # TODO: address hint error and remove ignore
-            #       error: Argument 1 to "append" of "list" has incompatible type "bytearray"; expected "bytes32"
-            #       [arg-type]
-            byte_array_tx.append(bytearray(coin.puzzle_hash))  # type: ignore[arg-type]
+            byte_array_tx.append(bytearray(coin.puzzle_hash))
         for coin in removals:
             tx_removals.append(coin.name())
-            # TODO: address hint error and remove ignore
-            #       error: Argument 1 to "append" of "list" has incompatible type "bytearray"; expected "bytes32"
-            #       [arg-type]
-            byte_array_tx.append(bytearray(coin.name()))  # type: ignore[arg-type]
+            byte_array_tx.append(bytearray(coin.name()))
 
         bip158: PyBIP158 = PyBIP158(byte_array_tx)
         encoded = bytes(bip158.GetEncoded())
@@ -1838,10 +1803,9 @@ def create_test_foliage(
         assert foliage_transaction_block is not None
 
         foliage_transaction_block_hash: Optional[bytes32] = foliage_transaction_block.get_hash()
-        # TODO: address hint error and remove ignore
-        #       error: Argument 1 has incompatible type "Optional[bytes32]"; expected "bytes32"  [arg-type]
+        assert foliage_transaction_block_hash is not None
         foliage_transaction_block_signature: Optional[G2Element] = get_plot_signature(
-            foliage_transaction_block_hash,  # type: ignore[arg-type]
+            foliage_transaction_block_hash,
             reward_block_unfinished.proof_of_space.plot_public_key,
         )
         assert foliage_transaction_block_signature is not None
@@ -1864,9 +1828,6 @@ def create_test_foliage(
     return foliage, foliage_transaction_block, transactions_info
 
 
-# TODO: address hint error and remove ignore
-#       error: Incompatible default for argument "seed" (default has type "bytes", argument has type "bytes32")
-#       [assignment]
 def create_test_unfinished_block(
     constants: ConsensusConstants,
     sub_slot_start_total_iters: uint128,
@@ -1883,7 +1844,7 @@ def create_test_unfinished_block(
     signage_point: SignagePoint,
     timestamp: uint64,
     blocks: BlockchainInterface,
-    seed: bytes32 = b"",  # type: ignore[assignment]
+    seed: bytes = b"",
     block_generator: Optional[BlockGenerator] = None,
     aggregate_sig: G2Element = G2Element(),
     additions: Optional[List[Coin]] = None,
@@ -1932,7 +1893,7 @@ def create_test_unfinished_block(
 
     new_sub_slot: bool = len(finished_sub_slots) > 0
 
-    cc_sp_hash: Optional[bytes32] = slot_cc_challenge
+    cc_sp_hash: bytes32 = slot_cc_challenge
 
     # Only enters this if statement if we are in testing mode (making VDF proofs here)
     if signage_point.cc_vdf is not None:
@@ -1955,10 +1916,8 @@ def create_test_unfinished_block(
                 rc_sp_hash = curr.finished_reward_slot_hashes[-1]
         signage_point = SignagePoint(None, None, None, None)
 
-    # TODO: address hint error and remove ignore
-    #       error: Argument 1 has incompatible type "Optional[bytes32]"; expected "bytes32"  [arg-type]
     cc_sp_signature: Optional[G2Element] = get_plot_signature(
-        cc_sp_hash,  # type: ignore[arg-type]
+        cc_sp_hash,
         proof_of_space.plot_public_key,
     )
     rc_sp_signature: Optional[G2Element] = get_plot_signature(rc_sp_hash, proof_of_space.plot_public_key)
