@@ -103,7 +103,7 @@ class TestCoinStoreWithBlocks:
                             mempool_mode=False,
                             height=softfork_height,
                         )
-                        tx_removals, tx_additions = tx_removals_and_additions(npc_result.npc_list)
+                        tx_removals, tx_additions = tx_removals_and_additions(npc_result.conds)
                     else:
                         tx_removals, tx_additions = [], []
 
@@ -161,22 +161,38 @@ class TestCoinStoreWithBlocks:
                 if block.is_transaction_block():
                     removals: List[bytes32] = []
                     additions: List[Coin] = []
+                    async with db_wrapper.write_db():
+                        if block.is_transaction_block():
+                            assert block.foliage_transaction_block is not None
+                            await coin_store.new_block(
+                                block.height,
+                                block.foliage_transaction_block.timestamp,
+                                block.get_included_reward_coins(),
+                                additions,
+                                removals,
+                            )
 
-                    if block.is_transaction_block():
-                        assert block.foliage_transaction_block is not None
-                        await coin_store.new_block(
-                            block.height,
-                            block.foliage_transaction_block.timestamp,
-                            block.get_included_reward_coins(),
-                            additions,
-                            removals,
-                        )
-
-                    coins = block.get_included_reward_coins()
-                    records = [await coin_store.get_coin_record(coin.name()) for coin in coins]
+                        coins = block.get_included_reward_coins()
+                        records = [await coin_store.get_coin_record(coin.name()) for coin in coins]
 
                     await coin_store._set_spent([r.name for r in records], block.height)
 
+                    if len(records) > 0:
+                        for r in records:
+                            assert (await coin_store.get_coin_record(r.name)) is not None
+
+                        if cache_size > 0:
+                            # Check that we can't spend a coin twice in cache
+                            with pytest.raises(ValueError, match="Coin already spent"):
+                                await coin_store._set_spent([r.name for r in records], block.height)
+
+                            for r in records:
+                                coin_store.coin_record_cache.remove(r.name)
+
+                        # Check that we can't spend a coin twice in DB
+                        with pytest.raises(ValueError, match="Invalid operation to set spent"):
+                            await coin_store._set_spent([r.name for r in records], block.height)
+
                     records = [await coin_store.get_coin_record(coin.name()) for coin in coins]
                     for record in records:
                         assert record.spent
