@@ -1,5 +1,3 @@
-# flake8: noqa: F811, F401
-import asyncio
 import atexit
 import logging
 from secrets import token_bytes
@@ -8,7 +6,6 @@ from typing import List, Optional
 import pytest
 import pytest_asyncio
 
-from chia.consensus.block_record import BlockRecord
 from chia.consensus.blockchain import ReceiveBlockResult
 from chia.consensus.find_fork_point import find_fork_point_in_chain
 from chia.consensus.multiprocess_validation import PreValidationResult
@@ -20,12 +17,11 @@ from chia.protocols.timelord_protocol import NewInfusionPointVDF
 from chia.types.blockchain_format.sized_bytes import bytes32
 from chia.types.unfinished_block import UnfinishedBlock
 from chia.util.block_cache import BlockCache
-from tests.block_tools import get_signage_point, create_block_tools
 from chia.util.hash import std_hash
 from chia.util.ints import uint8, uint32, uint64, uint128
+from tests.block_tools import get_signage_point, create_block_tools
 from tests.blockchain.blockchain_test_utils import (
     _validate_and_add_block,
-    _validate_and_add_block_multi_result,
     _validate_and_add_block_no_error,
 )
 from tests.setup_nodes import test_constants as test_constants_original
@@ -49,18 +45,18 @@ log = logging.getLogger(__name__)
 
 @pytest_asyncio.fixture(scope="function", params=[1, 2])
 async def empty_blockchain(request):
-    bc1, connection, db_path = await create_blockchain(test_constants, request.param)
+    bc1, db_wrapper, db_path = await create_blockchain(test_constants, request.param)
     yield bc1
-    await connection.close()
+    await db_wrapper.close()
     bc1.shut_down()
     db_path.unlink()
 
 
 @pytest_asyncio.fixture(scope="function", params=[1, 2])
-async def empty_blockchain_original(request):
-    bc1, connection, db_path = await create_blockchain(test_constants_original, request.param)
+async def empty_blockchain_with_original_constants(request):
+    bc1, db_wrapper, db_path = await create_blockchain(test_constants_original, request.param)
     yield bc1
-    await connection.close()
+    await db_wrapper.close()
     bc1.shut_down()
     db_path.unlink()
 
@@ -426,10 +422,8 @@ class TestFullNodeStore:
         )
 
         # Get signage point by hash
-        # TODO: address hint error and remove ignore
-        #       error: Argument 1 to "get_signage_point" of "FullNodeStore" has incompatible type "Optional[bytes32]";
-        #       expected "bytes32"  [arg-type]
-        assert store.get_signage_point(saved_sp_hash) is not None  # type: ignore[arg-type]
+        assert saved_sp_hash is not None
+        assert store.get_signage_point(saved_sp_hash) is not None
         assert store.get_signage_point(std_hash(b"2")) is None
 
         # Test adding signage points before genesis
@@ -627,7 +621,7 @@ class TestFullNodeStore:
         for block in blocks:
             await _validate_and_add_block_no_error(blockchain, block)
 
-        log.warning(f"Starting loop")
+        log.warning("Starting loop")
         while True:
             log.warning("Looping")
             blocks = bt.get_consecutive_blocks(1, block_list_input=blocks, skip_slots=1)
@@ -739,8 +733,8 @@ class TestFullNodeStore:
         await self.test_basic_store(empty_blockchain, True)
 
     @pytest.mark.asyncio
-    async def test_long_chain_slots(self, empty_blockchain_original, default_1000_blocks):
-        blockchain = empty_blockchain_original
+    async def test_long_chain_slots(self, empty_blockchain_with_original_constants, default_1000_blocks):
+        blockchain = empty_blockchain_with_original_constants
         store = FullNodeStore(test_constants_original)
         blocks = default_1000_blocks
         peak = None
