@@ -31,7 +31,6 @@ class TestHintStore:
 
             hints = [(coin_id_0, hint_0), (coin_id_1, hint_0), (coin_id_2, hint_1)]
             await hint_store.add_hints(hints)
-            await db_wrapper.commit_transaction()
             coins_for_hint_0 = await hint_store.get_coin_ids(hint_0)
 
             assert coin_id_0 in coins_for_hint_0
@@ -54,7 +53,6 @@ class TestHintStore:
 
             hints = [(coin_id_0, hint_0), (coin_id_0, hint_1)]
             await hint_store.add_hints(hints)
-            await db_wrapper.commit_transaction()
             coins_for_hint_0 = await hint_store.get_coin_ids(hint_0)
             assert coin_id_0 in coins_for_hint_0
 
@@ -73,7 +71,6 @@ class TestHintStore:
 
             hints = [(coin_id_0, hint_0), (coin_id_1, hint_0)]
             await hint_store.add_hints(hints)
-            await db_wrapper.commit_transaction()
             coins_for_hint_0 = await hint_store.get_coin_ids(hint_0)
             assert coin_id_0 in coins_for_hint_0
             assert coin_id_1 in coins_for_hint_0
@@ -91,12 +88,12 @@ class TestHintStore:
             for i in range(0, 2):
                 hints = [(coin_id_0, hint_0), (coin_id_0, hint_0)]
                 await hint_store.add_hints(hints)
-                await db_wrapper.commit_transaction()
             coins_for_hint_0 = await hint_store.get_coin_ids(hint_0)
             assert coin_id_0 in coins_for_hint_0
 
-            cursor = await db_wrapper.db.execute("SELECT COUNT(*) FROM hints")
-            rows = await cursor.fetchall()
+            async with db_wrapper.read_db() as conn:
+                cursor = await conn.execute("SELECT COUNT(*) FROM hints")
+                rows = await cursor.fetchall()
 
             if db_wrapper.db_version == 2:
                 # even though we inserted the pair multiple times, there's only one
@@ -160,7 +157,6 @@ class TestHintStore:
             coin_id_1 = 32 * b"\5"
             hints = [(coin_id_0, hint_0), (coin_id_1, hint_1)]
             await hint_store.add_hints(hints)
-            await db_wrapper.commit_transaction()
 
             count = await hint_store.count_hints()
             assert count == 2
