@@ -73,7 +73,7 @@ async def check_spend_bundle_validity(
     `SpendBundle`, and then invokes `receive_block` to ensure that it's accepted (if `expected_err=None`)
     or fails with the correct error code.
     """
-    connection, blockchain = await create_ram_blockchain(constants)
+    db_wrapper, blockchain = await create_ram_blockchain(constants)
     try:
         for block in blocks:
             await _validate_and_add_block(blockchain, block)
@@ -98,8 +98,8 @@ async def check_spend_bundle_validity(
         return coins_added, coins_removed
 
     finally:
-        # if we don't close the connection, the test process doesn't exit cleanly
-        await connection.close()
+        # if we don't close the db_wrapper, the test process doesn't exit cleanly
+        await db_wrapper.close()
 
         # we must call `shut_down` or the executor in `Blockchain` doesn't stop
         blockchain.shut_down()
