@@ -10,7 +10,7 @@ from tests.util.temp_file import TempFile
 from chia.types.blockchain_format.sized_bytes import bytes32
 from chia.util.ints import uint32, uint64
 from chia.cmds.db_upgrade_func import convert_v1_to_v2
-from chia.util.db_wrapper import DBWrapper
+from chia.util.db_wrapper import DBWrapper2
 from chia.full_node.block_store import BlockStore
 from chia.full_node.coin_store import CoinStore
 from chia.full_node.hint_store import HintStore
@@ -54,13 +54,13 @@ class TestDbUpgrade:
 
         with TempFile() as in_file, TempFile() as out_file:
 
-            async with aiosqlite.connect(in_file) as conn:
+            conn = await aiosqlite.connect(in_file)
+            await conn.execute("pragma journal_mode=OFF")
+            await conn.execute("pragma synchronous=OFF")
 
-                await conn.execute("pragma journal_mode=OFF")
-                await conn.execute("pragma synchronous=OFF")
-                await conn.execute("pragma locking_mode=exclusive")
-
-                db_wrapper1 = DBWrapper(conn, 1)
+            db_wrapper1 = DBWrapper2(conn, 1)
+            await db_wrapper1.add_connection(await aiosqlite.connect(in_file))
+            try:
                 block_store1 = await BlockStore.create(db_wrapper1)
                 coin_store1 = await CoinStore.create(db_wrapper1, uint32(0))
                 if with_hints:
@@ -73,20 +73,27 @@ class TestDbUpgrade:
                 bc = await Blockchain.create(
                     coin_store1, block_store1, test_constants, hint_store1, Path("."), reserved_cores=0
                 )
-                await db_wrapper1.commit_transaction()
 
                 for block in blocks:
                     # await _validate_and_add_block(bc, block)
                     results = PreValidationResult(None, uint64(1), None, False)
                     result, err, _, _ = await bc.receive_block(block, results)
                     assert err is None
+            finally:
+                await db_wrapper1.close()
 
             # now, convert v1 in_file to v2 out_file
             convert_v1_to_v2(in_file, out_file)
 
-            async with aiosqlite.connect(in_file) as conn, aiosqlite.connect(out_file) as conn2:
+            conn = await aiosqlite.connect(in_file)
+            db_wrapper1 = DBWrapper2(conn, 1)
+            await db_wrapper1.add_connection(await aiosqlite.connect(in_file))
 
-                db_wrapper1 = DBWrapper(conn, 1)
+            conn2 = await aiosqlite.connect(out_file)
+            db_wrapper2 = DBWrapper2(conn2, 2)
+            await db_wrapper2.add_connection(await aiosqlite.connect(out_file))
+
+            try:
                 block_store1 = await BlockStore.create(db_wrapper1)
                 coin_store1 = await CoinStore.create(db_wrapper1, uint32(0))
                 if with_hints:
@@ -94,7 +101,6 @@ class TestDbUpgrade:
                 else:
                     hint_store1 = None
 
-                db_wrapper2 = DBWrapper(conn2, 2)
                 block_store2 = await BlockStore.create(db_wrapper2)
                 coin_store2 = await CoinStore.create(db_wrapper2, uint32(0))
                 hint_store2 = await HintStore.create(db_wrapper2)
@@ -133,3 +139,6 @@ class TestDbUpgrade:
                     for c in coins:
                         n = c.coin.name()
                         assert await coin_store1.get_coin_record(n) == await coin_store2.get_coin_record(n)
+            finally:
+                await db_wrapper1.close()
+                await db_wrapper2.close()
